package prime.toolbox.gui;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;

//import prime.net.dataobjects.IData;
import prime.game.dataobjects.Data;
import prime.toolbox.scomm.SComm;
import prime.toolbox.scomm.SCommException;

import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;

import java.util.ArrayList;

/**
 * This class provides the functionality of a dialog box used to change 
 * values of a PRIME game objects’ properties. It implements four methods, to
 * create the dialog, depending on the nature of the value that is going to be
 * changed (either a sequence of values, a simple (primitive) property or a
 * PRIME server object).
 * 
 * 
 * @author      Aleksandar Dimov
 * @version     2.0, 2007-06-25
 */

public class ChangeValueDialog {

	private Shell		dialog			= null; 
	private Display 	display			= null;
	private Shell		parentShell		= null;

	private String		newStringValue	= null;
//	private IData 		newIData		= null;
//	private IData[] 	newArrayIData	= null;
	private Object[] 	newArrayObject	= null;
	
	private boolean isAdd;

	/**
     * Constructs a new instance of this class.
     * 
     * @param	display		parent display for the dialog
     */

	public ChangeValueDialog( Display display ) {
	
		this.display = display;
		
		dialog = new Shell( display );
		dialog.setText( "Enter new value" );

		FormLayout dialog_form = new FormLayout ();
		dialog_form.marginWidth = dialog_form.marginHeight = 4;
		dialog_form.spacing = 15;
		dialog.setLayout( dialog_form );
	}

	/**
     * Constructs a new instance of this class.
     * 
     * @param	display		parent display for the dialog
     * @param	title		Dialog title 
     * 
     */

	public ChangeValueDialog( Display display, String title ) {
		
		this.display = display;
		
		dialog = new Shell( display, SWT.NONE );
		dialog.setText( title );

		FormLayout dialog_form = new FormLayout ();
		dialog_form.marginWidth = dialog_form.marginHeight = 4;
		dialog_form.spacing = 15;
		dialog.setLayout( dialog_form );
	}
	
	/**
     * Constructs a new instance of this class.
     * 
     * @param	display		parent display for the dialog
     * @param	shell		parent shell for the dialog
     * @param	title		Dialog title
     * @param	style		SWT style options for the dialog 
     * 
     */

	public ChangeValueDialog( Display display, Shell shell, String title, int style ) {
		this.display = display;
		this.parentShell = shell;
		
		dialog = new Shell( display, style );
		shell.setParent( parentShell );
		dialog.setText( title );

		FormLayout dialog_form = new FormLayout ();
		dialog_form.marginWidth = dialog_form.marginHeight = 4;
		dialog_form.spacing = 15;
		dialog.setLayout( dialog_form );
	}

	/**
     * Visualizes modal dialog window and places its controls at the
     * appropriate places. This method is intended for usage by the <code>
     * ToolboxObjectViewer</code> class. It declares two objects of anonymous
     * inner classes that add listeners to the OK and Cancel buttons. The
     * dialog waits for user input and closes if either of the two buttons is
     * pressed. If the OK button is pressed, then the function returns a String,
     * containing the new value of the property.
     * 
     * @param	theOldValue 	The current value of the property
     * @return					The new value of the property		 
     */

	protected String CreateDialog ( String theOldValue ) {

		Button	okButton		= new Button( dialog, SWT.PUSH );
		Button	cancelButton	= new Button( dialog, SWT.PUSH );
		okButton.setText( "&OK" );
		cancelButton.setText( "&Cancel" );

		Label oldValueLabel = new Label( dialog, SWT.RIGHT );
		oldValueLabel.setText( "Old Value:" );
		Label newValueLabel = new Label( dialog, SWT.RIGHT );
		newValueLabel.setText( "New Value:" );
		
		Text oldValue = new Text( dialog, SWT.LEFT );
		oldValue.setText( theOldValue );
		oldValue.setEditable( false );
		final Text newValue = new Text( dialog, SWT.LEFT );
		newValue.setFocus();
		
		okButton.addListener( SWT.Selection, new Listener(){
			public void handleEvent(Event arg0) {
				newStringValue = newValue.getText();
				dialog.dispose();
			}
		});
		
		cancelButton.addListener( SWT.Selection, new Listener(){
			public void handleEvent(Event arg0) {
				dialog.dispose();
			}
		});
		
		FormData oldValueLabel_data = new FormData();
		oldValueLabel_data.top = new FormAttachment( 0, 0 );
		oldValueLabel_data.left = new FormAttachment( 0, 0 );
		oldValueLabel.setLayoutData( oldValueLabel_data );
		
		FormData newValueLabel_data = new FormData();
		newValueLabel_data.top = new FormAttachment( oldValueLabel );
		newValueLabel.setLayoutData( newValueLabel_data );
		
		FormData oldValue_data = new FormData();
		oldValue_data.left = new FormAttachment( oldValueLabel );
		oldValue.setLayoutData( oldValue_data );
	
		FormData newValue_data = new FormData();
		newValue_data.top = new FormAttachment( oldValue );
		newValue_data.left = new FormAttachment( newValueLabel );
		newValue.setLayoutData( newValue_data );
		
		FormData okButton_data = new FormData();
		okButton_data.top = new FormAttachment( 80, -15 );
		okButton_data.left = new FormAttachment( 20, 0 );
		okButton.setLayoutData( okButton_data );
		
		FormData cancelButton_data = new FormData();
		cancelButton_data.top = new FormAttachment( 80, -15 );
		cancelButton_data.left = new FormAttachment( 60, 0 );
		cancelButton.setLayoutData( cancelButton_data );

		dialog.setSize( 300, 140 );
		dialog.setDefaultButton ( okButton );
		dialog.open();
		
		while (!dialog.isDisposed ()) {
			if (!display.readAndDispatch ()) display.sleep ();
			else
				if ( display.isDisposed() ) {
					dialog.dispose();
					return null;
				}
		}
		
		return newStringValue;
	}
	
	/**	
     * Visualizes the dialog window and places its controls at the appropriate
     * places. This method is intended for usage by the <code>
     * ToolboxObjectViewer</code> class.Returns an IData reference for an
     * object that should be used as a new value for the selected property.
     * Visualizes a modal dialog window, declares two objects of anonymous
     * inner classes that add listeners to the OK and Cancel buttons.
     * 
	 * @param dataObjectClassName	Class name of the PRIME server data object
	 * 								that is going to be modified.
	 * @param objectName			TOM name of the object, as defined by the
	 * 								<code>name</code> attribute of the <code>
	 * 								object</code> tag.
	 * 
	 * @return						The object that is selected by the user to
	 * 								be used as a new value for the property.
	 */
	
	protected Data CreateDialog ( String dataObjectClassName, final String objectName ) {
//		final IData []objects = SComm.get_serverObject().getObjectList( dataObjectClassName );
//		final Combo comboBox = new Combo ( dialog, SWT.None );
//		
//		try {
//			for (int i = 0; i < objects.length; i++ ) {
//				SComm comm = ObjectsView.getComm();
//					comboBox.add( comm.getCivilName( objects[i], objectName ) );
//			}
//		} catch (SCommException e) {
//			e.printStackTrace();
//		}
//
//		Button	okButton		= new Button( dialog, SWT.PUSH );
//		Button	cancelButton	= new Button( dialog, SWT.PUSH );
//		okButton.setText( "&OK" );
//		cancelButton.setText( "&Cancel" );
//
//		FormData okButton_data = new FormData();
//		okButton_data.top = new FormAttachment( 80, -15 );
//		okButton_data.left = new FormAttachment( 20, 0 );
//		okButton.setLayoutData( okButton_data );
//		
//		FormData cancelButton_data = new FormData();
//		cancelButton_data.top = new FormAttachment( 80, -15 );
//		cancelButton_data.left = new FormAttachment( 60, 0 );
//		cancelButton.setLayoutData( cancelButton_data );
//
//		FormData comboBox_data = new FormData();
//		comboBox_data.top = new FormAttachment(10, 0);
//		comboBox_data.left = new FormAttachment(20, 0);
//		comboBox.setLayoutData(comboBox_data);
//
//		okButton.addListener( SWT.Selection, new Listener(){
//			public void handleEvent(Event arg0) {
//				int ind = comboBox.getSelectionIndex();
//				if ( ind >= 0 )
//					newIData = objects[ind];
//				dialog.dispose();
//			}
//		});
//		
//		cancelButton.addListener( SWT.Selection, new Listener(){
//			public void handleEvent(Event arg0) {
//				dialog.dispose();
//			}
//		});
//
//		dialog.setSize( 300, 140 );
//		dialog.setDefaultButton ( okButton );
//		dialog.open();
//		
//		while (!dialog.isDisposed ()) {
//			if (!display.readAndDispatch ()) display.sleep ();
//			else
//				if ( display.isDisposed() ) {
//					dialog.dispose();
//					return null;
//				}
//		}
//
//		return newIData;
		return null;
	}
	
	/**
	 * Returns an array of <code>IData</code> objects that is going to be
	 * used as a new value for the user-selected (within the <code>ObjectsView
	 * </code> class sequence property. This method is intended for usage by
	 * the <code>ToolboxObjectViewer</code> class. This method implements two 
	 * radio-buttons, and declares anonymous inner classes that add listeners
	 * to them. The radio-buttons are used to determine, whether the current
	 * property sequence is going to be extended or reduced by one element.
	 * 
	 * @param objectName			TOM name of the object, as defined by the
	 * 								<code>name</code> attribute of the <code>
	 * 								object</code> tag.
	 * @param propertyObjectData	The current sequence of property values. 
	 * @return						The modified sequence of property values
	 */
	protected Data[] CreateDialogAddDelIData (  final String objectName, ArrayList<Object> propertyObjectData ) {
//		final IData []allObjects = SComm.get_serverObject().getObjectList( objectName.substring(1) );
//		final ArrayList<IData> currentObjects = new ArrayList(propertyObjectData.size());
//		final ArrayList<IData> comboBoxObjects = new ArrayList();
//				
//		for (int j=0;j<propertyObjectData.size(); j++)
//			currentObjects.add( (IData) propertyObjectData.get(j) );			
//		
//		final Combo comboBox = new Combo ( dialog, SWT.None );
//		
//		final Button	okButton		= new Button( dialog, SWT.PUSH );
//		okButton.setText( "&OK" );
//		
//		Button	cancelButton	= new Button( dialog, SWT.PUSH );
//		cancelButton.setText( "&Cancel" );
//		
//		Button addBtn = new Button(dialog, SWT.RADIO);
//		addBtn.setText("Add");
//		addBtn.addFocusListener(new FocusAdapter(){
//			public void focusGained(FocusEvent e){
//				System.out.println("add");
//				isAdd = true;
//				comboBox.removeAll();
//				comboBoxObjects.clear();
//				okButton.setText("Add");
//				try {
//	    			for (int i = 0; i < allObjects.length; i++ ) {
//	    				SComm comm = ObjectsView.getComm();
//	    				if (!currentObjects.contains(allObjects[i])){
//	    					comboBox.add( comm.getCivilName( allObjects[i], objectName ) );
//	    					comboBoxObjects.add(allObjects[i]);
//	    				}
//	    			}
//	    		} catch (SCommException es) {
//	    			es.printStackTrace();
//	    		}
//			}
//		});		
//	
//		Button delBtn = new Button(dialog, SWT.RADIO);
//		delBtn.setText("Del");
//		delBtn.addFocusListener(new FocusAdapter(){
//			public void focusGained(FocusEvent e){
//				System.out.println("del");	
//				isAdd = false;
//				comboBox.removeAll();	
//				comboBoxObjects.clear();
//				okButton.setText("Del");
//				try {
//					for (int i=0; i<currentObjects.size(); i++){
//						SComm comm = ObjectsView.getComm();
//						comboBox.add(comm.getCivilName( currentObjects.get(i), objectName ));
//						comboBoxObjects.add(currentObjects.get(i));
//					}
//				}catch (SCommException es) {
//	    			es.printStackTrace();
//	    		}					
//			}
//		});				
//		
//		FormData addButton_data = new FormData();
//		addButton_data.top = new FormAttachment( 10, 0 );
//		addButton_data.left = new FormAttachment( 10, 0 );
//		addBtn.setLayoutData( addButton_data );
//		
//		FormData delButton_data = new FormData();
//		delButton_data.top = new FormAttachment( 30, 0 );
//		delButton_data.left = new FormAttachment( 10, 0 );
//		delBtn.setLayoutData( delButton_data );
//
//		FormData okButton_data = new FormData();
//		okButton_data.top = new FormAttachment( 80, -15 );
//		okButton_data.left = new FormAttachment( 20, 0 );
//		okButton.setLayoutData( okButton_data );
//		
//		FormData cancelButton_data = new FormData();
//		cancelButton_data.top = new FormAttachment( 80, -15 );
//		cancelButton_data.left = new FormAttachment( 60, 0 );
//		cancelButton.setLayoutData( cancelButton_data );
//
//		FormData comboBox_data = new FormData();
//		comboBox_data.top = new FormAttachment(10, 0);
//		comboBox_data.left = new FormAttachment(40, 0);
//		comboBox.setLayoutData(comboBox_data);
//
//		okButton.addListener( SWT.Selection, new Listener(){
//			public void handleEvent(Event arg0) {
//				int index = comboBox.getSelectionIndex();
//				if(index != -1) {
//					if (isAdd){
//						IData newObject = comboBoxObjects.get(index);
//						currentObjects.add(newObject);
//						System.out.println(objectName);
//						newArrayIData = new IData[currentObjects.size()];
//						for (int k=0; k<currentObjects.size(); k++)
//							newArrayIData[k] = currentObjects.get(k);	
//						dialog.dispose();
//					}
//					else{
//						IData delObject = comboBoxObjects.get(index);
//						currentObjects.remove(delObject);
//						newArrayIData = new IData[currentObjects.size()];
//						for (int k=0; k<currentObjects.size(); k++)
//							newArrayIData[k] = currentObjects.get(k);	
//						dialog.dispose();
//					}					
//				}	
//				else{
//					MessageBox box = new MessageBox(dialog, SWT.OK | SWT.ICON_INFORMATION );
//					box.setText( "PRIME Toolbox" );
//					box.setMessage( "Please choose an object." );
//					box.open();
//				}				
//			}
//		});
//		
//		cancelButton.addListener( SWT.Selection, new Listener(){
//			public void handleEvent(Event arg0) {
//				dialog.dispose();
//			}
//		});
//
//		dialog.setSize( 300, 160 );
//		dialog.setDefaultButton ( addBtn );
//		dialog.open();
//		
//		while (!dialog.isDisposed ()) {
//			if (!display.readAndDispatch ()) display.sleep ();
//			else
//				if ( display.isDisposed() ) {
//					dialog.dispose();
//					return null;
//				}
//		}		
//		return newArrayIData;
		return null;
	}
	
	/**
	 * Returns an array of non-<code>IData</code> objects that is going to be
	 * used as a new value for the user-selected (within the <code>ObjectsView
	 * </code> class sequence property. This method is intended for usage by
	 * the <code>ToolboxObjectViewer</code> class. This method implements two 
	 * radio-buttons, and declares anonymous inner classes that add listeners
	 * to them. The radio-buttons are used to determine, whether the current
	 * property sequence is going to be extended or reduced by one element.
	 * 
	 * @param objectName			TOM name of the object, as defined by the
	 * 								<code>name</code> attribute of the <code>
	 * 								object</code> tag.
	 * @param propertyObjectData	The current sequence of property values. 
	 * @return						The modified sequence of property values
	 */
	protected Object[] CreateDialogAddDelPrimitiveType (  final String objectName, ArrayList<Object> propertyObjectData ) {
	
		final ArrayList<Object> currentObjects = new ArrayList(propertyObjectData.size());
		final ArrayList<Object> comboBoxObjects = new ArrayList();
				
		for (int j=0;j<propertyObjectData.size(); j++)
			currentObjects.add( propertyObjectData.get(j) );			
		
		final Combo comboBox = new Combo ( dialog, SWT.None );		
		final Text textField = new Text(dialog, SWT.SINGLE | SWT.BORDER);
		
		final Button	okButton		= new Button( dialog, SWT.PUSH );
		okButton.setText( "&OK" );
		
		Button	cancelButton	= new Button( dialog, SWT.PUSH );
		cancelButton.setText( "&Cancel" );
			
		Button addBtn = new Button(dialog, SWT.RADIO);
		addBtn.setText("Add");
		addBtn.addFocusListener(new FocusAdapter(){
			public void focusGained(FocusEvent e){
				System.out.println("add");
				isAdd = true;
				comboBox.setVisible(false);
				textField.setVisible(true);
				okButton.setText("Add");
				
				comboBoxObjects.clear();
				
	    		for (int i = 0; i < currentObjects.size(); i++ ) {
	    			comboBoxObjects.add(currentObjects.get(i));
	    		}    		
			}
		});		
	
		Button delBtn = new Button(dialog, SWT.RADIO);
		delBtn.setText("Del");
		delBtn.addFocusListener(new FocusAdapter(){
			public void focusGained(FocusEvent e){
				System.out.println("del");	
				isAdd = false;
				comboBox.setVisible(true);
				textField.setVisible(false);
				okButton.setText("Del");
				comboBox.removeAll();	
				comboBoxObjects.clear();
				for (int i=0; i<currentObjects.size(); i++){
					String value = currentObjects.get(i).toString();
					comboBox.add(value);
					comboBoxObjects.add(currentObjects.get(i));
				}								
			}
		});				

		okButton.addListener( SWT.Selection, new Listener(){
			public void handleEvent(Event arg0) {
				if (isAdd && textField.getText() != ""){
					Object newObject = (Object) textField.getText();
					currentObjects.add(newObject);					
					newArrayObject = new Object[currentObjects.size()];
					for (int k=0; k<currentObjects.size(); k++)
						newArrayObject[k] = currentObjects.get(k);	
					dialog.dispose();
				}
				else if (comboBox.getSelectionIndex() != -1){
					Object delObject = comboBoxObjects.get(comboBox.getSelectionIndex());
					currentObjects.remove(delObject);
					newArrayObject = new Object[currentObjects.size()];
					for (int k=0; k<currentObjects.size(); k++)
						newArrayObject[k] = currentObjects.get(k);
					dialog.dispose();					
				}
				else {
					MessageBox box = new MessageBox(dialog, SWT.OK | SWT.ICON_INFORMATION );
					box.setText( "PRIME Toolbox" );
					box.setMessage( "Please choose an object or enter value." );
					box.open();
				}				
			}
		});
		
		cancelButton.addListener( SWT.Selection, new Listener(){
			public void handleEvent(Event arg0) {
				dialog.dispose();
			}
		});
		
		FormData addButton_data = new FormData();
		addButton_data.top = new FormAttachment( 10, 0 );
		addButton_data.left = new FormAttachment( 10, 0 );
		addBtn.setLayoutData( addButton_data );
		
		FormData delButton_data = new FormData();
		delButton_data.top = new FormAttachment( 30, 0 );
		delButton_data.left = new FormAttachment( 10, 0 );
		delBtn.setLayoutData( delButton_data );

		FormData okButton_data = new FormData();
		okButton_data.top = new FormAttachment( 80, -15 );
		okButton_data.left = new FormAttachment( 20, 0 );
		okButton.setLayoutData( okButton_data );
		
		FormData cancelButton_data = new FormData();
		cancelButton_data.top = new FormAttachment( 80, -15 );
		cancelButton_data.left = new FormAttachment( 60, 0 );
		cancelButton.setLayoutData( cancelButton_data );

		FormData comboBox_data = new FormData();
		comboBox_data.top = new FormAttachment(10, 0);
		comboBox_data.left = new FormAttachment(40, 0);
		comboBox.setLayoutData(comboBox_data);
		
		FormData textBox_data = new FormData();
		textBox_data.top = new FormAttachment(10, 0);
		textBox_data.left = new FormAttachment(40, 0);
		textField.setLayoutData(textBox_data);

		dialog.setSize( 300, 160 );
		dialog.setDefaultButton ( addBtn );
		dialog.open();
		
		while (!dialog.isDisposed ()) {
			if (!display.readAndDispatch ()) display.sleep ();
			else
				if ( display.isDisposed() ) {
					dialog.dispose();
					return null;
				}
		}	
		return newArrayObject;
	}
}
