package prime.toolbox.gui;

import java.util.*;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;

import prime.toolbox.ObjectCollection;
import prime.toolbox.ObjectDef;
import prime.toolbox.scomm.SComm;

/**
 * <code>CreateObjectDialog</code> class is used to create
 * dialog box for choosing an object.
 * 
 * @author      Stanisav Stoykov
 * @version     1.2, 2006-03-06
 */

public class ChooseObjectDialog {
	private Shell dialog = null; 
	private Display display = null;
	private Shell parentShell = null;
	
	private String returnValue = null;
		
	/**
     * Constructs a new instance of this class.
     * 
     * @param	display		Parent display for the dialog
     */
	public ChooseObjectDialog( Display display){
		this.display = display;		
		dialog = new Shell( display );
		dialog.setText( "Choose object" );
	}
	
	/**
     * Constructs a new instance of this class.
     * 
     * @param	display		Parent display for the dialog
     * @param	title		Dialog title 
     * 
     */	
	public ChooseObjectDialog( Display display, String title ) {
		
		this.display = display;		
		dialog = new Shell( display );
		dialog.setText( title );
	}
	
	/**
     * Constructs a new instance of this class.
     * 
     * @param	display		parent display for the dialog
     * @param	shell		parent shell for the dialog
     * @param	title		Dialog title
     * @param	style		SWT style options for the dialog 
     * 
     */
	public ChooseObjectDialog( Display display, Shell shell, String title, int style) {
		this.display = display;
		this.parentShell = shell;
		
		dialog = new Shell( display, style );
		shell.setParent( parentShell );
		dialog.setText( title );
	}
	
	/**
     * Visualizes the dialog window and places its controls at 
     * the appropriate places.
     * 
     * @return			The class name of the chosen object.		 
     */	
	public String CreateDialog()  {
	
		final ObjectCollection objects = SComm.getObjects();
		
		String[] names = new String[objects.size()];
		List<String> listNames = new ArrayList();
		
		ObjectDef object = null;
		int objIdx;
		
		for( objIdx = 0; objIdx < objects.size(); objIdx++)	{
			object = objects.getObjectDef(objIdx);
			listNames.add(object.getName());
		}
		Collections.sort(listNames);		
		
		for( objIdx = 0; objIdx < objects.size(); objIdx++)	{			
			names[objIdx] = listNames.get(objIdx);		
		}
		
		final Combo combo = new Combo (this.dialog, SWT.NONE);
		combo.setItems (names);		

		Button buttonOK = new Button(dialog, SWT.PUSH);
		buttonOK.setText("Ok");
		buttonOK.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				int index = combo.getSelectionIndex();
				if(index != -1) {
					String name = combo.getText();
					ObjectDef object = objects.getObjectDef(name);
					returnValue = object.getName();
					System.out.println(name + " " + returnValue);
					CreateObjectDialog crObj = new CreateObjectDialog(display, dialog, "Enter property values", SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM, returnValue);
					crObj.createDialog();
					if (CreateObjectDialog.isOkPressed)
						dialog.dispose();
				}
				else {
					MessageBox box = new MessageBox(dialog, SWT.OK | SWT.ICON_INFORMATION);
					box.setMessage("Please choose an object.");
					box.open();
				}				
			}
		});		
		
		Button buttonCancel = new Button(this.dialog, SWT.PUSH);
		buttonCancel.setText("Cancel");
		buttonCancel.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				dialog.dispose();				
			}
		});
		
		FormLayout dialog_form = new FormLayout ();
		dialog_form.marginWidth = dialog_form.marginHeight = 4;
		dialog_form.spacing = 15;
		dialog.setLayout( dialog_form );
		
		FormData comboBox_data = new FormData();
		comboBox_data.top = new FormAttachment(10, 0);
		comboBox_data.left = new FormAttachment(20, -10);
		combo.setLayoutData(comboBox_data);
		
		FormData buttonOK_data = new FormData();
		buttonOK_data.top = new FormAttachment( 80, -15 );
		buttonOK_data.left = new FormAttachment( 20, 0 );
		buttonOK.setLayoutData( buttonOK_data );
		
		FormData buttonCancel_data = new FormData();
		buttonCancel_data.top = new FormAttachment( 80, -15 );
		buttonCancel_data.left = new FormAttachment( 60, 0 );
		buttonCancel.setLayoutData( buttonCancel_data );
		
		this.dialog.setDefaultButton(buttonOK);
		
		dialog.setSize( 200, 120 );		
		combo.pack();
		
		dialog.open ();
		while (!dialog.isDisposed ()) {
			if (!display.readAndDispatch ()) display.sleep ();
		//	else
		//		if ( display.isDisposed() ) {
		//			dialog.dispose();
		//			return null;					
		//		}
		}
		
		return returnValue;		
	}

}
