package prime.toolbox.gui;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import prime.game.dataobjects.BusinessUnit;
import prime.toolbox.scomm.SComm;
import org.hibernate.Session;
/**
* <code>CreateBUDialog</code> class is used to create
* dialog box for creating a BusinessUnit
* 
* @author      Stanisav Stoykov
* @version     2.0, 2007-11-03
*/
public class CreateBUDialog {
	private Shell dialog = null; 
	private Display display = null;
	private Shell parentShell = null;
	
	private String objectName = null;
	
	private ArrayList<Object> objectData = null;
	private Object struct  = null;
	
	private SimpleDateFormat dateFormat = null;
		
	public static boolean isOkPressed = false;
	
	private String civilName = null;
		
	
	/**
     * Constructs a new instance of this class.
     * 
     * @param	display			Parent display for the dialog
     * @param	objectClassName	The class name of the object which will be created
     */
public CreateBUDialog( Display display, String objectName){
	
		this.display = display;	
		dialog = new Shell( this.display );
		
		dialog.setLayout(new GridLayout());
		
		dialog.setText( "Enter property values" );	
		
		this.objectName = objectName;
	}

/**
 * Constructs a new instance of this class.
 * 
 * @param	display			Parent display for the dialog
 * @param	title			Dialog title
 * @param	objectClassName	The class name of the object which will be created
 */
public CreateBUDialog( Display display, String title, String objectName ) {
	
	this.display = display;	
	
	dialog = new Shell( this.display );
	
	dialog.setLayout(new GridLayout());
	
	dialog.setText( title );
	
	this.objectName = objectName;
}

/**
 * Constructs a new instance of this class.
 * 
 * @param	display			parent display for the dialog
 * @param	shell			parent shell for the dialog
 * @param	title			Dialog title
 * @param	style			SWT style options for the dialog 
 * @param	objectClassName	The class name of the object which will be created
 * 
 */
public CreateBUDialog( Display display, Shell shell, String title, int style, String objectName) {
	this.display = display;
	this.parentShell = shell;
	
	dialog = new Shell( display, style );
	
	dialog.setLayout(new GridLayout());
	
	shell.setParent( parentShell );
	dialog.setText( title );
	
	this.objectName = objectName;
}


/**
 * Visualizes the dialog window and places its controls at 
 * the appropriate places.
 * 
 * @return			The created object.		 
 */	
public String createDialog() {
			
	ScrolledComposite sc = new ScrolledComposite(dialog, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
	sc.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
	
	Composite composite = new Composite(sc, SWT.NONE);
	GridLayout gritLayoutComposite = new GridLayout();
	gritLayoutComposite.numColumns = 2;	
	composite.setLayout(gritLayoutComposite);
		
	dateFormat = (SimpleDateFormat) SComm.getDateFormat();	
			
	Label lblMain = new Label(composite, SWT.BORDER);
	lblMain.setText("Enter property values for BusinessUnit object" );
	
	GridData gridData = new GridData(60,15);
	gridData.horizontalAlignment = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	lblMain.setLayoutData(gridData);
	
	Label lblValue = new Label(composite, SWT.NONE);

	Label lblName = new Label(composite, SWT.NONE);
	final Text textName = new Text(composite, SWT.SINGLE | SWT.BORDER);
	
	Label lblAmount = new Label(composite, SWT.NONE);
	final Text textAmount = new Text(composite, SWT.SINGLE | SWT.BORDER);
	
	lblName.setText("Enter Name (string)");
	lblAmount.setText("Enter Amount (int)");	
	
	
	lblName.setLayoutData(gridData);
	textName.setLayoutData(gridData);
	
	lblAmount.setLayoutData(gridData);
	textAmount.setLayoutData(gridData);	
	

	Button buttonOK = new Button(composite, SWT.PUSH);
	buttonOK.setText("Ok");
	
	buttonOK.addSelectionListener(new SelectionAdapter() {
		
		public void widgetSelected(SelectionEvent e) {

			if (checkInt(textAmount.getText())){
			
				BusinessUnit bu = new BusinessUnit();
				bu.setName(textName.getText());
				bu.setAmount(new Integer(textAmount.getText()));
			
				Session se = ToolboxObjectViewer.getSComm().getDataManager().openSession();
				bu.save(se);
				ToolboxObjectViewer.getSComm().getDataManager().closeSession(se);
						
				isOkPressed = true;
				try{
					civilName =  ToolboxObjectViewer.getSComm().getCivilName(bu, "BusinessUnit");
				}
				catch (Exception ex){
					System.out.println("BusinessUnit is not created correcly.");
				}
			
				MessageBox box = new MessageBox(dialog, SWT.OK | SWT.ICON_INFORMATION);
				box.setMessage("Object BusinessUnit was created.");
				box.setText("New object created.");
				box.open();
			
				dialog.dispose();
			}
		}
	});		
	
	Button buttonCancel = new Button(composite, SWT.PUSH);
	buttonCancel.setText("Cancel");
	buttonCancel.addSelectionListener(new SelectionAdapter() {
		@Override
		public void widgetSelected(SelectionEvent e) {
			isOkPressed = false;
			dialog.dispose();	
		}
	});	
	
	this.dialog.setDefaultButton(buttonOK);
	
	sc.setContent(composite);
	sc.setExpandHorizontal(true);
	sc.setExpandVertical(true);
	sc.setMinSize(composite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	
	dialog.setSize( 450, 180 );	
	
	dialog.open ();
	while (!this.dialog.isDisposed ()) {
		if (!display.readAndDispatch ()) display.sleep();
	}

	return civilName;
}

private boolean checkInt(String str){
	try{
		int number = new Integer(str);
		return true;
	}
	catch(Exception e){
		MessageBox box = new MessageBox(dialog, SWT.OK | SWT.ICON_INFORMATION);
		box.setMessage("Not correct value at Amount field");
		box.setText("Wrong type");
		box.open();
		return false;
	}
}
}
