package prime.toolbox.gui;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.GregorianCalendar;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import prime.game.dataobjects.Data;
import prime.toolbox.*;
import prime.toolbox.scomm.SComm;
import prime.toolbox.scomm.SCommException;
import prime.net.dataobjects.*;
import java.lang.reflect.Field;
/**
* <code>CreateStruct</code> class is used to create
* dialog box for setting the properties of a
* struct.
* 
* @author      Stanisav Stoykov
* @version     2.0, 2007-11-03
*/
public class CreateStructDialog {
	private Shell dialog = null; 
	private Display display = null;
	private Shell parentShell = null;
	
	private String objectName = null;
	
	private ArrayList<Object> objectData = null;
	private Object struct  = null;
	private ObjectDef object = null;
	private Text[] texts = null;
	private Combo[] combos = null;
	
	private SimpleDateFormat dateFormat = null;
		
	public static boolean isOkPressed = false;
	private IImage image = null;
	
	
	/**
     * Constructs a new instance of this class.
     * 
     * @param	display			Parent display for the dialog
     * @param	objectClassName	The class name of the object which will be created
     */
public CreateStructDialog( Display display, String objectName){
	
		this.display = display;	
		dialog = new Shell( this.display );
		
		dialog.setLayout(new GridLayout());
		
		dialog.setText( "Enter property values" );	
		
		this.objectName = objectName;
	}

/**
 * Constructs a new instance of this class.
 * 
 * @param	display			Parent display for the dialog
 * @param	title			Dialog title
 * @param	objectClassName	The class name of the object which will be created
 */
public CreateStructDialog( Display display, String title, String objectName ) {
	
	this.display = display;	
	
	dialog = new Shell( this.display );
	
	dialog.setLayout(new GridLayout());
	
	dialog.setText( title );
	
	this.objectName = objectName;
}

/**
 * Constructs a new instance of this class.
 * 
 * @param	display			parent display for the dialog
 * @param	shell			parent shell for the dialog
 * @param	title			Dialog title
 * @param	style			SWT style options for the dialog 
 * @param	objectClassName	The class name of the object which will be created
 * 
 */
public CreateStructDialog( Display display, Shell shell, String title, int style, String objectName) {
	this.display = display;
	this.parentShell = shell;
	
	dialog = new Shell( display, style );
	
	dialog.setLayout(new GridLayout());
	
	shell.setParent( parentShell );
	dialog.setText( title );
	
	this.objectName = objectName;
}


/**
 * Visualizes the dialog window and places its controls at 
 * the appropriate places.
 * 
 * @return			The created object.		 
 */	
public Object createDialog() {
	
	ScrolledComposite sc = new ScrolledComposite(dialog, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
	sc.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
	
	Composite composite = new Composite(sc, SWT.NONE);
	GridLayout gritLayoutComposite = new GridLayout();
	gritLayoutComposite.numColumns = 2;	
	composite.setLayout(gritLayoutComposite);
		
	dateFormat = (SimpleDateFormat) SComm.getDateFormat();
	
	final ObjectCollection objects = SComm.getObjects();
	object = objects.getObjectDef(objectName);
		
	Label lblMain = new Label(composite, SWT.BORDER);
	lblMain.setText("Enter property values for " + object.getName() + " object" );
	
	GridData gridData = new GridData(60,15);
	gridData.horizontalAlignment = GridData.FILL;
	gridData.grabExcessHorizontalSpace = true;
	lblMain.setLayoutData(gridData);
	
	Label lblValue = new Label(composite, SWT.NONE);
	
	int vPos = 40;
	final int objSize = object.size();
	
	objectData = new ArrayList<Object>(objSize);
	for (int i = 0; i < objSize; i++)
		objectData.add(null);
	
	Label[] labels = new Label[objSize];
	texts = new Text[objSize];
	combos = new Combo[objSize];
	
	for( int propIdx=0; propIdx<objSize; propIdx++)
	{
		PropertyDef prop = object.getPropertyDef(propIdx); 		
		
		labels[propIdx] = new Label(composite, SWT.NONE);
		if (prop.getType().equals("date"))
			labels[propIdx].setText("Enter " + prop.getName() + " (" + prop.getType() + " " + dateFormat.toPattern() + ")");
		else
			labels[propIdx].setText("Enter " + prop.getName() + " (" + prop.getType() + ")");
	
		labels[propIdx].setLayoutData(gridData);
		
		if (prop.isReadonly()){
			texts[propIdx] = new Text(composite, SWT.SINGLE | SWT.BORDER | SWT.Deactivate);
			texts[propIdx].setText("This is readonly field.");
			texts[propIdx].setLayoutData(gridData);
			combos[propIdx] = null;
		}
		else if (prop.isSeq()){
			texts[propIdx] = new Text(composite, SWT.SINGLE | SWT.BORDER | SWT.Deactivate);
			texts[propIdx].setText("This is a sequence field.");
			texts[propIdx].setLayoutData(gridData);
			combos[propIdx] = null;
		}
		else{
			String type = prop.getType();	
				
			// primitive type 
			if ( (!ObjectData.isObject(type))){
				texts[propIdx] = new Text(composite, SWT.SINGLE | SWT.BORDER);
				texts[propIdx].setLayoutData(gridData);
				combos[propIdx] = null;
			}
			// structure
			else if (ObjectData.isStruct(prop) == 1){
				combos[propIdx] = null;
				texts[propIdx] = null;
				
				Button createStruct = new Button(composite, SWT.PUSH);
				createStruct.setText("Create");
				createStruct.addSelectionListener(new SelectionAdapter() {
					public void widgetSelected(SelectionEvent e) {
						System.out.println("Not supported!!");						
					}
				});
			}
			// Data object
			else if (ObjectData.isObject(type)){
				texts[propIdx] = null;
				combos[propIdx] = new Combo (composite, SWT.READ_ONLY);
				String[] names = getNames(type);		
				combos[propIdx].setItems(names);
				combos[propIdx].setToolTipText("Choose id of the object " + type );
				combos[propIdx].setLayoutData(gridData);
			}
		}					
		vPos+=30;
	}	

	Button buttonOK = new Button(composite, SWT.PUSH);
	buttonOK.setText("Ok");
	
	final String className = this.objectName;
	buttonOK.addSelectionListener(new SelectionAdapter() {
		
		public void widgetSelected(SelectionEvent e) {
		//	check if all property fields are with correct values and saves them in an ArrayList.	
			if (createArrayList(objSize)){
			
				try{
					Class cl = Class.forName(object.getClassName());
					struct = cl.newInstance();
					Field fields[] = cl.getFields();
					for (int i = 0; i < fields.length; i++){						
						fields[i].set(struct, objectData.get(i));
					}					
				}
				catch(Exception exc){
					exc.printStackTrace();
				}	
			
				isOkPressed = true;
				dialog.dispose();	
			}				
		}
	});		
	
	Button buttonCancel = new Button(composite, SWT.PUSH);
	buttonCancel.setText("Cancel");
	buttonCancel.addSelectionListener(new SelectionAdapter() {
		@Override
		public void widgetSelected(SelectionEvent e) {
			isOkPressed = false;
			dialog.dispose();	
		}
	});	
	
	this.dialog.setDefaultButton(buttonOK);
	
	sc.setContent(composite);
	sc.setExpandHorizontal(true);
	sc.setExpandVertical(true);
	sc.setMinSize(composite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	
	if (vPos + 60 > 600){	
		dialog.setSize( 450, 620 );	
	}else{	
		dialog.setSize( 450, vPos+80 );	
	}
	
	dialog.open ();
	while (!this.dialog.isDisposed ()) {
		if (!display.readAndDispatch ()) display.sleep();
	}

	return struct;
}


/**
 * Fills the ArrayList <code>objectData</code> with 
 * the values form text fields and/or 
 * combo box objects. This ArrayList will be used to create 
 * the ObjectData object.
 * 
 * @return			true if all fields are filled corectly.	 
 */	
private boolean createArrayList(int objSize){
	
	for (int i = 0; i < objSize; i++ ){
		PropertyDef prop = object.getPropertyDef(i); 		
		String type = prop.getType();
		String propName = prop.getName();
		
		if ( prop.isReadonly() || prop.getSeq() != null)	{		
			continue;
		}
		else if (prop.getType().equals("IImage")){
			objectData.add(image);		
		}
		else if (ObjectData.isStruct(prop) == 1){
			continue;
		}
		else if (combos[i] == null && texts[i] != null){		
			if (texts[i].getText() == "" || texts[i].getText() == null){
			
			}
			
			else if (type.equals("long")) {
				try{
					long longValue = new Long(texts[i].getText());
					objectData.set(i, longValue);	
				}
				catch(Exception e){
					exeptionMessage(propName);
					return false;
				}		
			}
			else if (type.equals("int")){
				try{
					int intValue = new Integer(texts[i].getText());
					objectData.set(i, intValue);
				}
				catch(Exception e){
					exeptionMessage(propName);
					return false;
				}				
			}
			else if (type.equals("float")){
				try{
					float floatValue = new Float(texts[i].getText());
					objectData.set(i, floatValue);
				}
				catch(Exception e){
					exeptionMessage(propName);
					return false;
				}				
			}
			else if (type.equals("double")){
				try{
					double doubleValue = new Double(texts[i].getText());
					objectData.set(i, doubleValue);
				}
				catch(Exception e){
					exeptionMessage(propName);
					return false;
				}				
			}
			else if (type.equals("string") || type.equals("String")){
				objectData.set(i, texts[i].getText());
			}
			else if (type.equals("date")){
				GregorianCalendar longDate = new GregorianCalendar();
				try {
					longDate.setTime( dateFormat.parse( texts[i].getText() ) );
				} catch (ParseException e) {
					System.err.println( "Error in parsing date: " + texts[i].getText() );
					exeptionMessage(propName);
					return false;
				}				
				objectData.set(i, longDate.getTimeInMillis());			
			}			
			
		}
		else{				
			if (combos[i].getSelectionIndex() == -1){
					
				//	MessageBox box = new MessageBox(dialog, SWT.OK | SWT.ICON_INFORMATION);
				//	box.setMessage("Please choose an object " + type);
				//	box.open();
				//	return false;
			}
			else {
		
				String civil1 = combos[i].getText();
				String civil2 = "";
				SComm scomm = ToolboxObjectViewer.getSComm();
							
				Data[] objects = scomm.getObjectList(type);
				Data ddata = null;		
				
				for (int j = 0; j < objects.length; j++){
					try{
						civil2 = scomm.getCivilName(objects[j], type);
					}
					catch (SCommException e){
						e.printStackTrace();
					}	
					if (civil2.equals(civil1))
					{						
						ddata = objects[j];
						System.out.println("civil: " + civil2 + " id: " + ddata.getId());
						break;
					}
				}							
				objectData.set(i, ddata); 
			}
		}		
	}	
	return true;
}

private void exeptionMessage(String propName){
	MessageBox box = new MessageBox(dialog, SWT.OK | SWT.ICON_INFORMATION);
	box.setMessage("Not correct value at " + propName + " field");
	box.setText("Wrong type");
	box.open();
}

/**
 * Creates an array of civil names of all objects of a given type. 
 * 
 * @param		type	The type of the object.
 * @return				Array of Strings.
 */	
private String[] getNames(String type){

	SComm scomm = ToolboxObjectViewer.getSComm();
	
	Data[] objects = scomm.getObjectList(type);
	System.out.println(type);
	String[] s = new String[objects.length];
	for(int i = 0; i < objects.length; i++){			
		try{
			s[i] = scomm.getCivilName(objects[i], type);
		}
		catch (SCommException e){
			e.printStackTrace();
		}	
	//	if there is no civil name, write the id.	
		if (s[i] == null)
			s[i] = "" + objects[i].getId();
	}
	return s;
}


}
