package prime.toolbox.gui;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.*;

import prime.game.dataobjects.Data;
import prime.net.IPrimeServer;
import prime.game.dataobjects.Player;
import prime.game.dataobjects.BusinessUnit;
import prime.game.dataobjects.PlayerProfile;
import prime.toolbox.scomm.SComm;
import prime.toolbox.scomm.SCommException;
import org.hibernate.Session;

/**
 * <code>CreateUserDialog</code> class is used to create
 * a dialog for creating new user.
 *  
 * @author      Stanisav Stoykov
 * @version     2.0, 2007-11-09
 */

public class CreateUserDialog {
	private Shell dialog = null; 
	private Display display = null;
	private Shell parentShell = null;	
	
	/**
     * Constructs a new instance of this class.
     * 
     * @param	display		Parent display for the dialog
     */
	public CreateUserDialog(Display display){
		this.display = display;	
		dialog = new Shell( display );
		dialog.setText( "Enter user name and password" );	
	}
	
	/**
     * Constructs a new instance of this class.
     * 
     * @param	display		parent display for the dialog
     * @param	shell		parent shell for the dialog
     * @param	title		Dialog title
     * @param	style		SWT style options for the dialog 
     * 
     */
	public CreateUserDialog( Display display, Shell shell, String title, int style ) {
		this.display = display;
		this.parentShell = shell;
		
		dialog = new Shell( display, style );
		shell.setParent( parentShell );
		dialog.setText( title );		
	}
	
	/**
     * Visualizes the dialog window and places its controls at 
     * the appropriate places.     
     */	
	public void createDialog(){
		
		Label lblUser = new Label(dialog, SWT.LEFT);
		Label lblPass = new Label(dialog, SWT.LEFT);
		Label chsBU = new Label(dialog, SWT.LEFT);
		
		final Text txtUser = new Text(dialog, SWT.SINGLE | SWT.LEFT);
		final Text txtPass = new Text(dialog, SWT.SINGLE | SWT.LEFT);
		
		lblUser.setText("Username:");
		lblUser.setBounds(10, 10, 60, 20);
		
		lblPass.setText("Password:");
		lblPass.setBounds(10, 40, 60, 20);
		
		chsBU.setText("Choose BU:");
		chsBU.setBounds(10, 70, 60, 20);
		
		txtUser.setBounds(80, 10, 160, 20);
		txtPass.setBounds(80, 40, 160, 20);
		
		final Combo combo = new Combo(dialog, SWT.READ_ONLY);
		String[] businessUnitNames = getNames("BusinessUnit");
		combo.setItems(businessUnitNames);
		combo.setBounds(80, 70, 80, 20);
		
		Button btnCreateBU = new Button(dialog, SWT.PUSH);
		btnCreateBU.setText("Create BU");
		btnCreateBU.setBounds(180, 70, 60, 20);
		btnCreateBU.addSelectionListener(new SelectionAdapter() {
			
			public void widgetSelected(SelectionEvent e) {	
				System.out.println("sss");
				CreateBUDialog crBU = new CreateBUDialog(display, dialog, "Enter property values", SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM, "BusinessUnit");
				String civilName = crBU.createDialog();
				if (civilName != null){
					System.out.println(civilName);
					combo.add(civilName);
				}
			}
		});
		
		Button btnOk = new Button(dialog, SWT.PUSH);
		btnOk.setText("Ok");
		btnOk.setBounds(10, 120, 50, 20);
		btnOk.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {														
				if (txtUser.getText() == "" || txtPass.getText() == ""){
					MessageBox box = new MessageBox(dialog, SWT.OK | SWT.ICON_INFORMATION);
					box.setMessage("Please fill the text field.");
					box.setText("Empty field");
					box.open();
				} else{	
					IPrimeServer _serverObject = SComm.get_serverObject();
					
					if (_serverObject != null){
					//	create new account and save it.	
						int newPlayerId = _serverObject.createAccount(txtUser.getText(), txtPass.getText());
							
						if (newPlayerId != 0){
							
							if (combo.getSelectionIndex() != -1){
								String civil1 = combo.getText();
								String civil2 = "";
								SComm scomm = ToolboxObjectViewer.getSComm();
											
								Data[] objects = scomm.getObjectList("BusinessUnit");
								BusinessUnit bu = null;		
								
								for (int j = 0; j < objects.length; j++){
									try{
										civil2 = scomm.getCivilName(objects[j], "BusinessUnit");
									}
									catch (SCommException ex){
										ex.printStackTrace();
									}	
									if (civil2.equals(civil1))
									{						
										bu = (BusinessUnit) objects[j];
										break;
									}
								}	
								Session se = scomm.getDataManager().openSession();
								PlayerProfile pp = new PlayerProfile();
								pp.setUserId(newPlayerId);
								pp.setBusinessUnitId(bu.getId());
								
								pp.save(se);
											
								scomm.getDataManager().closeSession(se);
						
								MessageBox box = new MessageBox(dialog, SWT.OK | SWT.ICON_INFORMATION);
								
								box.setMessage("New user created with id: " + newPlayerId + ".");
								box.setText("New user created.");
								box.open();
								
								dialog.dispose();
							}
							else{
								MessageBox box = new MessageBox(dialog, SWT.OK | SWT.ICON_INFORMATION);
								
								box.setMessage("Plese select a BusinessUnit.");
								box.setText("");
								box.open();
							}					
						}
						else{
							MessageBox box = new MessageBox(dialog, SWT.OK | SWT.ICON_INFORMATION);
							
							box.setMessage("Unable to create new user.");
							box.setText("");
							box.open();
							
							dialog.dispose();
						}
						
						
						
					}
					else {				
						System.out.println("No connection with the server.");
						dialog.dispose();
					}
				}								
			}
		});		
		
		Button btnCancel = new Button(this.dialog, SWT.PUSH);
		btnCancel.setText("Cancel");
		btnCancel.setBounds(90, 120, 50, 20);
		btnCancel.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				dialog.dispose();					
			}
		});	
		
		dialog.setSize( 260, 190 );			
		dialog.open ();
		while (!this.dialog.isDisposed ()) {
			if (!display.readAndDispatch ()) display.sleep();
		}
		
		return;
		
	}
	
	private String[] getNames(String type){

		SComm scomm = ToolboxObjectViewer.getSComm();
		
		Data[] objects = scomm.getObjectList(type);
	
		String[] s = new String[objects.length];
		for(int i = 0; i < objects.length; i++){			
			try{
				s[i] = scomm.getCivilName(objects[i], type);
			}
			catch (SCommException e){
				e.printStackTrace();
			}	
		//	if there is no civil name, write the id.	
			if (s[i] == null)
				s[i] = "" + objects[i].getId();
		}
		return s;
	}

}
