package prime.toolbox.gui;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.lang.reflect.Field;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;

import prime.net.IPrimeServer;
import prime.client.PrimeClient;
import prime.game.dataobjects.Data;
import prime.toolbox.ObjectData;
import prime.toolbox.PropertyDef;
import prime.toolbox.scomm.SComm;
import prime.toolbox.scomm.SCommException;
import prime.toolbox.xml.importer.Importer;
import prime.toolbox.xml.importer.ImporterException;
import prime.toolbox.xml.reader.ReaderException;

import java.util.ArrayList;


public class ToolboxObjectViewer {
	
	/**
	 * <code>ToolboxObjectViewer</code> class is responsible to run the main thread 
	 * of the Prime toolbox and start the GUI of the application.
	 * 
	 * @author      Aleksandar Dimov
	 * @version     1.9, 2007-06-11
	 */

	private static final int CONTROL_SPACING = 15;
	private static final int FORM_WIDTH = 350;
	private static final int FORM_HEIGTH = 400;
	private static final String mess01 = "Type your username and password to start...";
	
	private static Display display = new Display();
	private static Shell shell = new Shell( display );
	private static boolean primeServerConnected = false;
	
	private static SComm	comm	= null;
	
	private static String TOM_File = null;
	
	public static void main(String[] args) {
		String address = "";
		int port = 0;
		try {
			address = args[0];
			port = Integer.parseInt(args[1]);
		} catch ( NumberFormatException e )	{
			port = 0;
		}
		
		String configFile = null;
	
		if ( args.length == 2 )
			configFile = "prime/prime/toolbox/xml/PrimeToolbox.cfg";
		else 
			configFile = args[2];
		
		String configLine = null;
		try{ 
			FileReader inFile = new FileReader(configFile);
			BufferedReader in = new BufferedReader(inFile);
			configLine = in.readLine();
		}
		catch(IOException e){
			System.out.println("Fatal error: cannot find the " + configFile + " file!");
//			e.printStackTrace();
		}
		
		if (configLine == null){
			System.err.println("Enter the name of TOM file at PrimeToolbox.cfg");
			return;
		}
		
		TOM_File = configLine;
		shell.setSize( FORM_WIDTH, FORM_HEIGTH );
		shell.setText( "Prime Toolbox" );
	
		final FillLayout layout = new FillLayout();
		
		FormLayout initial_view_layout = new FormLayout();
		initial_view_layout.spacing = CONTROL_SPACING;
		initial_view_layout.marginWidth = 3;
		initial_view_layout.marginHeight = 3;
		shell.setLayout(initial_view_layout);
		
		final Button login = new Button( shell, SWT.PUSH );
		login.setText("&Continue");
		shell.setDefaultButton( login );
		
		final Button exit = new Button( shell, SWT.PUSH );
		exit.setText("&Qiut");
	
		final Text inp_user = new Text( shell, SWT.LEFT );
		inp_user.setToolTipText( mess01 );
		inp_user.setFocus();
		final Text inp_pass = new Text( shell, SWT.LEFT );
		inp_pass.setToolTipText( mess01 );
		inp_pass.setEchoChar('*');
		
		final Label user = new Label (shell, SWT.LEFT );
		user.setText("Username:");
		final Label pass = new Label (shell, SWT.LEFT );
		pass.setText("Password:");
		
		final Label label_port = new Label ( shell, SWT.LEFT );
		label_port.setText( "Server port number:" );
		final Label label_address = new Label ( shell, SWT.LEFT );
		label_address.setText( "Server address:" );
		
		final Text text_port = new Text ( shell, SWT.LEFT );
		text_port.setText( new Integer(port).toString() );
		final Text text_address = new Text ( shell, SWT.LEFT );
		text_address.setText( address );
		
		FormData login_data = new FormData();
		login_data.left		= new FormAttachment( 30, 0 );
		login_data.right	= new FormAttachment( 50, 0 );
		login_data.top 		= new FormAttachment( 80, 0 );
		login_data.bottom 	= new FormAttachment( 90, 0 );
		login.setLayoutData( login_data );
				
		FormData exit_data	= new FormData();
		exit_data.left		= new FormAttachment( login );
		exit_data.right		= new FormAttachment( 70, CONTROL_SPACING );
		exit_data.top		= new FormAttachment( 80, 0 );
		exit_data.bottom	= new FormAttachment( 90, 0 );
		exit.setLayoutData( exit_data );
		
		FormData user_data	= new FormData();
		user_data.left		= new FormAttachment( 15, 0 );
		user_data.top		= new FormAttachment( 10, 0 );
		user.setLayoutData( user_data );
		
		FormData pass_data	= new FormData();
		pass_data.left		= new FormAttachment( 15, 0 );
		pass_data.top		= new FormAttachment( user );
		pass.setLayoutData( pass_data );
		
		FormData inp_user_data	= new FormData();
		inp_user_data.left		= new FormAttachment( user );
		inp_user_data.right		= new FormAttachment( 85, 0 );
		inp_user_data.top 		= new FormAttachment( 10, 0 );
		inp_user.setLayoutData( inp_user_data );
				
		FormData inp_pass_data	= new FormData();
		inp_pass_data.left		= new FormAttachment( pass );
		inp_pass_data.right		= new FormAttachment( 85, 0 );
		inp_pass_data.top		= new FormAttachment( inp_user );
		inp_pass.setLayoutData( inp_pass_data );

		FormData label_port_data	= new FormData();
		label_port_data.left		= new FormAttachment( 15, 0 );
		label_port_data.top			= new FormAttachment( pass );
		label_port.setLayoutData( label_port_data ); 
		
		FormData label_address_data = new FormData();
		label_address_data.left		= new FormAttachment( 15, 0 );
		label_address_data.top		= new FormAttachment( label_port );
		label_address.setLayoutData( label_address_data ); 
		
		FormData text_port_data = new FormData();
		text_port_data.left		= new FormAttachment( label_port );
		text_port_data.right	= new FormAttachment( 85, 0 );
		text_port_data.top		= new FormAttachment( inp_pass );
		text_port.setLayoutData( text_port_data );
		
		FormData text_address_data	= new FormData();
		text_address_data.left		= new FormAttachment( label_address );
		text_address_data.right		= new FormAttachment( 85, 0 );
		text_address_data.top		= new FormAttachment( text_port );
		text_address.setLayoutData( text_address_data );

		final Menu bar = new Menu( shell, SWT.BAR );
		MenuItem actionsItem = new MenuItem( bar, SWT.CASCADE );
		actionsItem.setText( "&Actions" );
		Menu actionsSubMenu = new Menu( shell, SWT.DROP_DOWN );
		actionsItem.setMenu( actionsSubMenu );
		
		final MenuItem createObject = new MenuItem( actionsSubMenu, SWT.PUSH );
		createObject.setText( "&Create new object" );
		
		final MenuItem LoadDB = new MenuItem( actionsSubMenu, SWT.PUSH );
		LoadDB.setText( "&Load database" );
		
		final MenuItem createUser = new MenuItem( actionsSubMenu, SWT.PUSH );
		createUser.setText("&Create new user");
		
		final MenuItem refresh = new MenuItem( actionsSubMenu, SWT.PUSH );
		refresh.setText("&Refresh");
		
		final MenuItem saveXML = new MenuItem( actionsSubMenu, SWT.PUSH );
		saveXML.setText("&Save database");
		
		final MenuItem modifyDataObjectProperty = new MenuItem ( actionsSubMenu, SWT.PUSH );
		modifyDataObjectProperty.setText( "&Modify" );
		
		final MenuItem loadKB = new MenuItem ( actionsSubMenu, SWT.PUSH );
		loadKB.setEnabled( true );
		loadKB.setText("&Load Knowledge Base");
		
		LoginButtonListener lbListener = new LoginButtonListener( bar, 
				login, 
				exit, 
				refresh, 
				pass, 
				user, 
				inp_pass, 
				modifyDataObjectProperty, 
				TOM_File, 
				createUser, 
				createObject, 
				saveXML, 
				layout, 
				inp_user, 
				LoadDB, 
				loadKB,
				label_address,
				label_port,
				text_address,
				text_port );
				
		login.addListener( SWT.Selection, lbListener );
		
		exit.addListener( SWT.Selection, new Listener() {

			public void handleEvent(Event arg0) {
				display.dispose();
			}
			
		});
		
		text_port.addListener( SWT.Modify, new Listener() {

			public void handleEvent(Event arg0) {
				try {
					@SuppressWarnings("unused")
					Integer vvv = new Integer( text_port.getText() );
				} catch ( NumberFormatException e ) {
					MessageBox box = new MessageBox(shell, SWT.OK | SWT.ICON_WARNING );
					box.setText( "Invalid Value" );
					box.setMessage( "Please, enter a correct port number" );
					box.open();
					text_port.setSelection( 0, text_port.getText().length() );
				}
			}
			
		} );
		MenuItem tomItem = new MenuItem( bar, SWT.CASCADE );
		tomItem.setText( "TOM" );
		Menu tomSubMenu = new Menu( shell, SWT.DROP_DOWN );
		tomItem.setMenu( tomSubMenu );

		final MenuItem tomInfo = new MenuItem( tomSubMenu, SWT.PUSH );
		tomInfo.setText( "&Display TOM information" );
		
		final MenuItem selectTOM = new MenuItem( tomSubMenu, SWT.PUSH );
		selectTOM.setText( "&Load TOM file" );
		
		final MenuItem generateTOM = new MenuItem( tomSubMenu, SWT.PUSH );
		generateTOM.setText( "&Generate TOM file" );	
		
		final MenuItem generateKB = new MenuItem( tomSubMenu, SWT.PUSH );
		generateKB.setText( "&Generate Knowledgebase" );	
		
		tomInfo.addSelectionListener( new SelectionListener(){

			public void widgetDefaultSelected(SelectionEvent arg0) {
				
			}

			public void widgetSelected(SelectionEvent arg0) {
				MessageBox box = new MessageBox( shell, SWT.OK | SWT.ICON_INFORMATION );
				box.setText( "TOM Info" );
				box.setMessage( "Currently working with ".concat( TOM_File ) );
				box.open();
			}
			
		});
		
		selectTOM.addSelectionListener( new SelectionListener(){

			public void widgetDefaultSelected(SelectionEvent arg0) {
				
			}

			public void widgetSelected(SelectionEvent arg0) {
				FileDialog openDialog = new FileDialog( shell, SWT.OPEN );
				openDialog.setFilterNames (new String [] {"PRIME TOM Files", "All Files (*.*)"});
				openDialog.setFilterExtensions (new String [] {"*.xml", "*.*"});
				
				String selectedFileName = openDialog.open();
				String file = "prime/prime/toolbox/xml/PrimeToolbox.cfg";
				
				if (selectedFileName != null){
					try{
						FileWriter outFile = new FileWriter(file);
						PrintWriter out = new PrintWriter(outFile);
						
						out.println(selectedFileName);
						out.flush();
						out.close();
					}
					catch (IOException e){
						e.printStackTrace();
					}
					MessageBox box = new MessageBox( shell, SWT.OK | SWT.ICON_WARNING );
					box.setText( "New TOM" );
					box.setMessage( "Please restart the Toolbox." );
					box.open();
				}				
			}
			
		});
		
		generateTOM.addSelectionListener( new SelectionListener(){
			public void widgetDefaultSelected(SelectionEvent arg0) {
				
			}
			
			public void widgetSelected(SelectionEvent arg0) {
				try{
					FileDialog saveDialog = new FileDialog(shell, SWT.SAVE );
					saveDialog.setText("Save");
					saveDialog.setFilterExtensions (new String [] {"*.xml", "*.*"});
					final String tomFileName = saveDialog.open();
					if (tomFileName != null){
						prime.toolbox.scomm.GenerateTOM.generate(tomFileName);
						
						MessageBox box = new MessageBox( shell, SWT.OK | SWT.ICON_WARNING );
						box.setText( "New TOM" );
						box.setMessage( "New TOM file was created." );
						box.open();
					}
				}
				catch(IOException e){
					e.printStackTrace();
					MessageBox box = new MessageBox( shell, SWT.OK | SWT.ICON_WARNING );
					box.setText( "TOM failed" );
					box.setMessage( "Unable to create TOM file." );
					box.open();
				}				
			}
			
		});
		
		generateKB.addSelectionListener( new SelectionListener(){
			public void widgetDefaultSelected(SelectionEvent arg0) {				
			}
			
			public void widgetSelected(SelectionEvent arg0) {
//				try{
					FileDialog saveDialog = new FileDialog(shell, SWT.SAVE );
					saveDialog.setText("Save");
					saveDialog.setFilterExtensions (new String [] {"*.alg", "*.*"});
					final String kbFileName = saveDialog.open();
					if (kbFileName != null){
//						prime.as.Demo.ProtegeProject.GenerateProtegeKnowledgeBase.generate(kbFileName);					
						System.out.println(kbFileName);
						
						MessageBox box = new MessageBox( shell, SWT.OK | SWT.ICON_WARNING );
						box.setText( "Knowledgebase" );
						box.setMessage( "The knowledgebase was successfully generated." );
						box.open();
					}
//				}
//				catch(IOException e){
//					e.printStackTrace();
//					MessageBox box = new MessageBox( shell, SWT.OK | SWT.ICON_WARNING );
//					box.setText( "Knowledgebase failed" );
//					box.setMessage( "Unable to generated the knowledgebase." );
//					box.open();
//				}
			}			
		});		

		shell.open();
		while ( !shell.isDisposed() ) {
			if ( !display.readAndDispatch() ) display.sleep();
		}
		display.dispose();
	}
	
	public static boolean checkAccount(String user, String pass, IPrimeServer _serverObject) {
			
//		IPlayer player = null;
//		if (_serverObject != null) {
//			/* IMPORTANT: if it is a process of login then should 
//			use getAccountLogin(user, pass), else 
//			getAccount(user, pass) is ok */
//			player = _serverObject.getAccountLogin(user, pass);
//			return (player != null);	
//		}		
//		else{	
//			return false;
//		}	
	return true;
	}
	
	public static SComm getSComm(){
		return comm;
	}

	/**
	 * <code>LoginButtonListener</code> class implements the main user
	 * functionality and is instantiated if there exist an accessible and
	 * running PRIME server. It loads the PRIME GOM browser associates
	 * listeners to user menus.
	 */
	private static final class LoginButtonListener implements Listener {
		private final Menu bar;

		private final Button login;

		private final Button exit;

		private final MenuItem refresh;

		private final Label pass;

		private final Label user;

		private final Text inp_pass;

		private final MenuItem modifyProperty;

		private final String file;

		private final MenuItem user2;

		private final MenuItem object;

		private final MenuItem savexml;

		private final FillLayout layout;

		private final Text inp_user;

		private final MenuItem loaddb;
		
		private final MenuItem loadKB;
		
		private final Label labelPort;
		
		private final Label labelAddress;
		
		private final Text textAddress;
		
		private final Text textPort;
		
		private int port;

		private String address;
		
		private LoginButtonListener(Menu bar, Button login, Button exit, MenuItem refresh, Label pass, Label user, Text inp_pass, MenuItem modifyProperty, String file, MenuItem user2, MenuItem object, MenuItem savexml, FillLayout layout, Text inp_user, MenuItem loaddb, MenuItem loadKB, Label label_address, Label label_port, Text text_address, Text text_port ) {
			this.bar = bar;
			this.login = login;
			this.exit = exit;
			this.refresh = refresh;
			this.pass = pass;
//			this.port = port;
			this.user = user;
			this.inp_pass = inp_pass;
			this.modifyProperty = modifyProperty;
//			this.address = address;
			this.file = file;
			this.user2 = user2;
			this.object = object;
			this.savexml = savexml;
			this.layout = layout;
			this.inp_user = inp_user;
			this.loaddb = loaddb;
			this.loadKB = loadKB;
			this.labelAddress = label_address;
			this.labelPort = label_port;			
			this.textAddress = text_address;
			this.textPort = text_port;
			this.address = text_address.getText();
			this.port = new Integer ( text_port.getText() ).intValue();
		}

		public void handleEvent( Event e ) {
			
			this.address = textAddress.getText();
			this.port = new Integer ( textPort.getText() ).intValue();

			try{
				comm = new SComm( file, address, port );
			} catch( ReaderException tbre ){
				System.out.println( tbre.getMessage() );
				tbre.printStackTrace();
			}
			if (comm != null){
				PrimeClient.setClient(comm);
				comm.run();
			}
			
		 	if ( checkAccount( inp_user.getText(), inp_pass.getText(), SComm.get_serverObject() ) ) {
		
				if ( !login.isDisposed() )		login.dispose();
				if ( !exit.isDisposed() )		exit.dispose();
				if ( !pass.isDisposed() )		pass.dispose();
				if ( !user.isDisposed() )		user.dispose();
				if ( !inp_pass.isDisposed() )	inp_pass.dispose();
				if ( !inp_user.isDisposed() )	inp_user.dispose();
				if ( !textAddress.isDisposed() )textAddress.dispose();
				if ( !textPort.isDisposed() )	textPort.dispose();
				if ( !labelPort.isDisposed() )	labelPort.dispose();
				if ( !labelAddress.isDisposed())labelAddress.dispose();
				
				shell.setLayout( layout );
			//	final ObjectsView vieww = new ObjectsView( shell, display );
				final ObjectsView vieww = new ObjectsView( shell, display, comm );
				primeServerConnected = vieww.makeTree(); // TOM_File );
				if ( primeServerConnected ) {
					loaddb.addListener( SWT.Selection, new Listener() {
						public void handleEvent(Event arg0) {
							try {
								//FileDialog should be created here, in order to load DataBase...
								FileDialog openDialog = new FileDialog( shell, SWT.OPEN );
								openDialog.setFilterNames (new String [] {"PRIME DataBase Files", "All Files (*.*)"});
								openDialog.setFilterExtensions (new String [] {"*.xml", "*.*"});
								openDialog.setFileName("prime/toolbox/xml/db.xml");
								String dbSelectedFileName = openDialog.open();
		
								if ( dbSelectedFileName != null) {
									@SuppressWarnings("unused")
									Importer xmlImporter = new Importer(
											TOM_File,	         // name of TOM file
											dbSelectedFileName ); // name of data file
								}
		
							} catch (ImporterException e) {
								e.printStackTrace();
							} catch (ReaderException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							} catch (SCommException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							} catch (IOException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
						}
					});
					
					user2.addListener( SWT.Selection, new Listener(){
						public void handleEvent( Event arg0 ) {
						//	CreateUserDialog dialog = new CreateUserDialog(display);
							CreateUserDialog dialog = new CreateUserDialog(display, shell, "Enter user name and password", SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM);
							dialog.createDialog();			
						}
					});
					
					object.addListener( SWT.Selection, new Listener(){
						public void handleEvent( Event arg0 ) {
							ChooseObjectDialog dialog = new ChooseObjectDialog(display, shell, "Choose object",SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM);
							String res;				
							res = dialog.CreateDialog();
							if (res == null){
								System.out.println("No class was selected.");
							}
							else
								System.out.println("It was selected: " + res );			
						}
					});
					
					refresh.addListener( SWT.Selection, new Listener() {
						public void handleEvent( Event arg0 ){
							System.out.println("Refresh menu choosed.");
							ObjectsView.getTree().removeAll();
							vieww.makeTree();							
						}
					});
					
					savexml.addListener( SWT.Selection, new Listener() {
						public void handleEvent( Event arg0 ){
							try{
								FileDialog saveDialog = new FileDialog(shell, SWT.SAVE );
								saveDialog.setText("Save");
								saveDialog.setFilterExtensions (new String [] {"*.xml", "*.*"});
								final String exporterFileName = saveDialog.open();
								if (exporterFileName != null){
									prime.toolbox.xml.exporter.Exporter.saveXml(exporterFileName);
									
									MessageBox box = new MessageBox( shell, SWT.OK | SWT.ICON_WARNING );
									box.setText( "Exporter" );
									box.setMessage( "The game was successfully saved." );
									box.open();
								}
							}
							catch(IOException e){
								e.printStackTrace();
								MessageBox box = new MessageBox( shell, SWT.OK | SWT.ICON_WARNING );
								box.setText( "Exporter failed" );
								box.setMessage( "Unable to save the game." );
								box.open();
							}													
						}
					});
					
					loadKB.addListener( SWT.Selection, new Listener() {
						public void handleEvent( Event arg0 ){
							System.out.println("Loading KB ...");
					
//							prime.as.Demo.Tenders.AlgernonTenderDialog asDialog = new prime.as.Demo.Tenders.AlgernonTenderDialog(display, shell, "AS", SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM);
						
//							asDialog.CreateDialog();							
						}
					});
					
					modifyProperty.addListener( SWT.Selection, new ModifyDataObjectAction() );
						
					shell.setMenuBar( bar );
					shell.setSize(600,400);
				} else {
					System.out.println( "Message Box: Server connection failed!" );
					shell.dispose();
				}
			}
			else
				System.out.println("Message Box: Login failed!");
		}
	}

	/**
	 * <code>ModifyDataObjectAction</code> class handles the manipulation of
	 * object properties, visualized by the <code>ObjcetsView</code> class. It
	 * defines single method, which checks whether the property is not defined
	 * as read-only in the TOM and considers different cases, depending the
	 * type of the property (primitive, <code>IData</code> or non-<code>IData
	 * </code> and sequence of values) as well as the type of its defining
	 * object (<code>IData</code> or non-<code>IData</code>).
	 */
	private static final class ModifyDataObjectAction implements Listener {

		public void handleEvent(Event arg0) {
			TreeItem item_sel = ObjectsView.getCurItemSel();
			if (item_sel == null) return;
		
			if( item_sel.getData() == null ) 
				return;
			
			try	{
				TreeItem parent1TreeItem = item_sel.getParentItem();
				ObjectData mainObjData = (ObjectData) parent1TreeItem.getData();
				System.out.println("mainObjData: " + mainObjData.getCivilName());
				String mainObjectClassName = mainObjData.getClassName();
				
				String propName = null;
				PropertyDef propDef = null;
				int i = 0;
				TreeItem items[] = parent1TreeItem.getItems();

				for ( i = 0; i < items.length; i++ ) {
					if ( items[i].equals( item_sel ) ) {
						propDef = mainObjData.getPropertyDef(i);
						if ( mainObjData.getPropertyDef( i ).isReadonly() ) {
							MessageBox box = new MessageBox(shell, SWT.OK | SWT.ICON_WARNING );
							box.setText( "Read-only field" );
							box.setMessage( "Can't set this field by the toolbox!" );
							box.open();
							return;
						}
						break;
					}
				}
				
				propName = propDef.getName();
				if ( propName == null ) {
					System.err.println( "Property does not exist!!!" );
					return;
				}

				// Modify properties of a structure (non IData) object 
				if ( Class.forName( mainObjectClassName ).getDeclaredMethods().length == 0 ) {
					// The object that contains the structure as a property
					// is three levels up in the tree hierarchy. It is needed
					// in order to get reference to the struct.
					Object structObj = mainObjData.getServerObjectReference();
					if ( structObj == null ) return;

					Field f = Class.forName( mainObjectClassName ).getDeclaredField( propDef.getName() );
					
					if ( !mainObjData.isObject( i ) ) {
						// Ask for a new value
						ChangeValueDialog prompt = new ChangeValueDialog( display, shell
								, "Change " + item_sel.getText(0)
								, SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM );
						String newvalue = prompt.CreateDialog( item_sel.getText( 1 ) );

						if ( propDef.getType().equals("long") )
							f.set( structObj, new Integer(newvalue).intValue() );
						else
							f.set( structObj, newvalue );

						item_sel.setText( 1, newvalue );
					} else {
						// To do modification of struct IData and other struct fields
					}

					return;
				} 
				
				// Modify a primitive IData object property
				if ( !mainObjData.isObject( i ) && !mainObjData.isSequence(i)) {
					// Ask for a new value
					ChangeValueDialog prompt = new ChangeValueDialog( display, shell
							, "Change " + item_sel.getText(0)
							, SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM );
					String newvalue = prompt.CreateDialog( item_sel.getText( 1 ) ); 

					if( newvalue != null ) {
						if ( ObjectsView.getComm().setPropertyValue( mainObjData
									, mainObjData.getPropertyDef( i )
									, newvalue ) )
							item_sel.setText( 1, newvalue );
						else {
							MessageBox box = new MessageBox(shell, SWT.OK | SWT.ICON_ERROR );
							box.setText( "Error" );
							box.setMessage( "Invalid value entered" );
							box.open();
							return;
						}
					} // end of if( newvalue != null )

				// Modify a single IData object property (non-array) 
				} else if ( !mainObjData.isSequence( i ) ) { // end of if ( !mainObjData.isObject( i ) )
					Data mainObject = null;
					
					try {
						 mainObject = (Data)mainObjData.getServerObjectReference();
						 // If the object is still null exception will occur
						 mainObject.getId();
					} catch ( Exception e ) {
						String errMess = mainObjData.getCivilName() + " ObjectData does not contain" +
									" valid IData object reference";
						MessageBox box = new MessageBox(shell, SWT.OK | SWT.ICON_ERROR );
						box.setText( "Unkown error" );
						box.setMessage( errMess );
						box.open();
						return;
					}

//					SComm.get_serverObject().getDataObject(id.intValue(), mainObjData.getClassName() );
					ChangeValueDialog prompt = new ChangeValueDialog( display, shell
						, "Select PRIME Data Object", SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM );
					ObjectData propertyObjectData = (ObjectData) item_sel.getData();
					if ( propertyObjectData == null ) {
						MessageBox box = new MessageBox(shell, SWT.OK | SWT.ICON_ERROR );
						box.setText( "Unkown error" );
						box.setMessage( "Selected Item does not contain valid data object" );
						box.open();
						return;
					}
					PropertyDef propertyDef = mainObjData.getPropertyDef(i);
					String type = propertyDef.getType();
					System.out.println("type: " + type);
					
					Data userNewObjectsel = prompt.CreateDialog( propertyObjectData.getClassName(), propertyObjectData.getName() );
					Data []newObject = null;
					if ( userNewObjectsel != null)
						newObject = new Data[]{ userNewObjectsel };
					
					if ( ( newObject != null ) && mainObjData.setPropertyIData( i, newObject[0] ) ) {
						if ( comm.setPropetyObjectValue( mainObject, mainObjData.getName(), newObject, propName ) ) {
							MessageBox box = new MessageBox(shell, SWT.OK | SWT.ICON_INFORMATION );
							box.setText( "PRIME Toolbox" );
							box.setMessage( "Please refresh the view to see the changes" );
							box.open();
						}
						else {
							MessageBox box = new MessageBox(shell, SWT.OK | SWT.ICON_INFORMATION );
							box.setText( "Unkown error" );
							box.setMessage( "Selected Item does not contain valid data object" );
							box.open();
							return;
						}
					}
				} else { // Modify an array IData object property
					
					if (mainObjData.isObject(i)){  //sequence with IData objects
						ChangeValueDialog prompt = new ChangeValueDialog( display, shell
								, "Select PRIME Data Object", SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM );
				//		ArrayList<Object> propertyObjectData = (ArrayList<Object>) item_sel.getData();
						ArrayList<Object> propertyObjectData = mainObjData.getPropertySequData(i);
						PropertyDef propertyDef = mainObjData.getPropertyDef(i);
						String type = propertyDef.getType();
						
						if ( propertyObjectData == null ) {
							MessageBox box = new MessageBox(shell, SWT.OK | SWT.ICON_ERROR );
							box.setText( "Unkown error" );
							box.setMessage( "Selected Item does not contain valid data objects" );
							box.open();
							return;
						}
						
						Data[] arrayToSet = prompt.CreateDialogAddDelIData( type, propertyObjectData );
						Data mainObject = (Data) mainObjData.getServerObjectReference();
											
					
						if (comm.setIDataArray(mainObject, arrayToSet, propertyDef)){
			//			if (prime.toolbox.test.TestSet.setBOMtestData(mainObject, arrayToSet, propertyDef)){
							MessageBox box = new MessageBox(shell, SWT.OK | SWT.ICON_INFORMATION );
							box.setText( "PRIME Toolbox" );
							box.setMessage( "Please refresh the view to see the changes" );
							box.open();
						}
						else{
							MessageBox box = new MessageBox(shell, SWT.OK | SWT.ICON_INFORMATION );
							box.setText( "Unkown error" );
							box.setMessage( "Selected Item does not contain valid data object" );
							box.open();
							return;
						}   							
						   									
					}
				// sequance but with primitive types	
					else {						
						ChangeValueDialog prompt = new ChangeValueDialog( display, shell
								, "Select PRIME Data Object", SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM );
						ArrayList<Object> propertyObjectData = (ArrayList<Object>) item_sel.getData();
						
						PropertyDef propertyDef = mainObjData.getPropertyDef(i);
						String type = propertyDef.getType();
						if ( propertyObjectData == null ) {
							MessageBox box = new MessageBox(shell, SWT.OK | SWT.ICON_ERROR );
							box.setText( "Unkown error" );
							box.setMessage( "Selected Item does not contain valid data object" );
							box.open();
							return;
						}
					
						Object[] objectsToSet = prompt.CreateDialogAddDelPrimitiveType(type, propertyObjectData);					
						Data mainObject = (Data) mainObjData.getServerObjectReference();
				
						if (comm.setPrimitiveArray(mainObject, objectsToSet, mainObjData, propertyDef)){	
							MessageBox box = new MessageBox(shell, SWT.OK | SWT.ICON_INFORMATION );
							box.setText( "PRIME Toolbox" );
							box.setMessage( "Please refresh the view to see the changes" );
							box.open();
						}
						else{
							MessageBox box = new MessageBox(shell, SWT.OK | SWT.ICON_INFORMATION );
							box.setText( "Unkown error" );
							box.setMessage( "Selected Item does not contain valid data object" );
							box.open();
							return;
						}   
						
					}
				}
			} catch ( ClassCastException e ) {
				System.err.println( "Parent tree item does not contain valid" +
						" ObjectData or the object data does not contain valid" +
						" PRIME server object reference..." );
				e.printStackTrace();
			} catch (SCommException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SecurityException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (NoSuchFieldException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalArgumentException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
}
