package prime.toolbox.scomm;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Date;
import java.util.GregorianCalendar;
import prime.net.IPrimeServer;
import prime.client.ClientDataManager;
import prime.client.PrimeBaseClient;
import prime.game.dataobjects.Data;
import prime.toolbox.ObjectCollection;
import prime.toolbox.ObjectData;
import prime.toolbox.ObjectDef;
import prime.toolbox.PropertyDef;
import prime.toolbox.xml.reader.Reader;
import prime.toolbox.xml.reader.ReaderException;
import prime.game.dataobjects.*;
import prime.game.dataobjects.finance.*;
import prime.game.dataobjects.production.*;
import prime.game.dataobjects.distribution.*;
import prime.game.dataobjects.development.*;
import prime.game.dataobjects.sales.*;
import prime.net.DataObject;
import org.hibernate.Session;
import org.omg.CORBA.ORB;
import prime.game.dataobjects.VBE;
import prime.client.PrimeClient;
import prime.net.dataobjects.IImage;
import prime.net.dataobjects.IImageHelper;
import java.lang.reflect.Array;

/**
 * <code>SComm</code> is responsible to manage the communication with the PRIME
 * server. It provides functionality to all other modules within the PRIME
 * Toolbox.
 * 
 * @author      Aleksandar Dimov
 * @version     2.0, 2007-02-26
 */

public class SComm extends PrimeBaseClient{

	private static ObjectCollection	objects			= null;
	private static DateFormat		dateFormat		= null;

	private static IPrimeServer 	_serverObject	= null;
	private static ORB 				_orb			= null;
	private static VBE				_vbe			= null;
	private static String			_hostAdress		= "localhost";
	private static int				_portNumber		= 99;


	public SComm( String tomFilename, String hostAddress, int portNum ) throws ReaderException	{
		Reader xmlReader = new Reader( tomFilename );
		objects = xmlReader.getObjects();
		dateFormat = new SimpleDateFormat( xmlReader.getDateFormat() );
		this._hostAdress = hostAddress;
		this._portNumber = portNum;
	}
	
	public void run() {
		
		connectionManager.connectToServer( _hostAdress, _portNumber );
		
		_serverObject = connectionManager.getServerObject();
		_vbe = dataManager.getVBE();
		_orb = connectionManager.orb;		
	}
	
	public Data[] getObjectList(String className) {
		Session se = dataManager.openSession();
		try {
			List<Data> datas = dataManager.queryData(se, "from " + className);
			return datas.toArray(new Data[0]);
		} catch (Exception e) {
			return null;
		} finally {
			dataManager.closeSession(se);
		}
	}


	public boolean isConnected(){
		if ( _serverObject == null ) return false;
		return true;
	}
	
	/**
	 * @return Returns the _vbe.
	 */
	public static VBE get_vbe() {
		if ( _serverObject == null ) return null;
		else return _vbe;
	}
	
	/**
	 * @return Returns the _serverObject.
	 */
	public static IPrimeServer get_serverObject() {
		return _serverObject;
	}
	
	/**
	 * @return Returns the _orb.
	 */
	public static ORB get_orb() {
		return _orb;
	}

	/**
	 * @return Returns the objects.
	 */
	public static ObjectCollection getObjects() {
		if ( _serverObject == null ) return null;
		else return objects;
	}

	public static DateFormat getDateFormat() {
		if ( _serverObject == null ) return null;
		else return dateFormat;
	}
	
	public String date2String( long date ) {
		return dateFormat.format( new Date( date ) );
	}
	
	public long string2Date( String date ) {
		GregorianCalendar longDate = new GregorianCalendar();
		try {
			longDate.setTime( dateFormat.parse( date ) );
		} catch (ParseException e) {
			System.err.println( "Error in parsing date: " + date );
			e.printStackTrace();
			return -1;
		}		
		return longDate.getTimeInMillis();
	}

	/**
	 * @return	Returns civil name of the PRIME data object given with
	 * 			the <code>IData dataObject</code> parameter. If the object
	 * 			does not have properties, defined as "civil" in the TOM
	 * 			file, this method returns <code>null</code>.
	 */

	public String getCivilName ( Data dataObject, String objectName ) throws SCommException {
		ObjectDef def = objects.getObjectDef( objectName );
		if( def == null )
			throw new SCommException( "Object " + objectName + " not defined!" );
		
		if ( dataObject == null ) return null; // Throw exception
		String civilName = "";
		boolean hasCivilName = false;
		
		for ( int i = 0; i < def.size(); i++) {
			PropertyDef prop = def.getPropertyDef( i );
			String propName = prop.getName();
			String propValue = ""; 

			if ( ( !ObjectData.isObject( prop.getType() )&& ( prop.isCivil() ) ) ) {
				if ( def.getName().equals("Region") &&
						( ( propName.toLowerCase().equals("population") 
						 || propName.toLowerCase().equals("numberofworkers") ) ) ) {
					Region region = (Region) dataObject;
					propValue = "" + region.getValue( propName );							
				}
				else {
					propValue = dataObject.getFieldValue( propName );
				}

				civilName += ( " " + propValue );
				hasCivilName = true;
			}
		}
		
		if ( hasCivilName )
			return civilName;
		else 
			return null;
	}
	
	public ObjectData retrieveVBEData() {
		try {
			return retrieveObject(_vbe, "VBE");
		} catch (SCommException e) {
			System.err.println( e.getMessage() );
			return null;
		}
	}
	
	public ObjectData retrieveImage(Object anObject){
		if ( anObject == null ) return null;
		
		ObjectDef def = objects.getObjectDef( "IImage" );
		if( def == null ) {
			System.err.println( "Object IImage not defined!" );
			return null;
		}
		ArrayList<Object> obj = new ArrayList<Object>();

		String mainObjectPackage = ObjectData.getPackageName( def );
		String mainObjectName = mainObjectPackage.concat(def.getName());
			
		IImage image = IImageHelper.narrow( (org.omg.CORBA.Object) anObject);
			
		try{
			String propValue = null;
			
			Class iImageClass = Class.forName(mainObjectName);
			for ( int i = 0; i < def.size(); i++) {
				PropertyDef prop = def.getPropertyDef( i );
				String propName = prop.getName();
			
				for (Method m : iImageClass.getMethods()){
					if (m.getName().equals("get".concat(propName))){
						propValue = m.invoke(image, new Object[0]).toString();
						obj.add(propValue);
						break;
					}						
				}		
			}			
			
		}catch(Exception e){
		//	e.printStackTrace();
			System.out.println("Exception in retrieveImage method. (File not found)");
			return null;
		}
		
		ObjectData retrObject = new ObjectData( def, obj, anObject );
	//	if ( hasCivilName )
	//		retrObject.setCivilName( civilName );
		return retrObject;
	}
	
	public ObjectData retrieveObject( Object anObject, String objectName ) throws SCommException {
		if ( anObject == null ) return null;

		// Determine if the object being retrieved is inherited from an abstract Data class
		if ( objectName.equals( "FunctionalUnit" ) ) {
			objectName = anObject.getClass().getSimpleName();	
		} else if ( objectName.equals( "OperationProcess" ) ) {
			objectName = anObject.getClass().getSimpleName();	
		} else if ( objectName.equals( "Clause" ) ) {
			objectName = anObject.getClass().getSimpleName();	
		}

		ObjectDef def = objects.getObjectDef( objectName );
		if( def == null ) {
			System.err.println( "Object " + objectName + " not defined!" );
			return null;
		}

		// Check if the object being retrieved is simple structure or Data
		// later in the code of this method only the objectIsStruct variable should be used.
		boolean objectIsStruct = false;
		try {
			objectIsStruct = Class.forName( def.getClassName() ).getDeclaredMethods().length == 0;

			// Check if the object is valid IData object, to prevent multiple
			// examinations in the rest of the method
			if ( !objectIsStruct ) {
				@SuppressWarnings("unused")
				Data dummy = ( Data )anObject;
			}
		} catch (SecurityException e1) {
			e1.printStackTrace();
			return null;
		} catch (ClassNotFoundException e1) {
			System.err.println( "Attempt to access non-existing object '" +
					def.getName() + "'." );
			return null;
		} catch ( ClassCastException cce ) {
			System.err.println( "SComm.retrieveObject(): Invalid IData object of type: '" + objectName + "'." );
			return null;
		}

		ArrayList<Object> obj = new ArrayList<Object>();
		String civilName = "";
		boolean hasCivilName = false;
		
		for ( int i = 0; i < def.size(); i++) {
			PropertyDef prop = def.getPropertyDef( i );
			String propName = prop.getName();

			try{
				// Object property contains an array (sequence) of values
				if ( ObjectData.isSequence( prop ) ) {

					// Object property is array of structures
					if ( ObjectData.isStruct( prop ) == 1 ) {
						Object propValue = null;

						if ( objectIsStruct ) {
							// Currently no structures have array fields so do nothing
						}
						else{
							propValue = getStructNew((Data) anObject, prop.getName());
						//	propValue = getStruct( (Data) anObject, def, prop.getName() );
						}

						obj.add( propValue );
					}
					else if ( ObjectData.isObject( prop.getType() ) ) {
						// Object property is array of another IData objects
						Data[] propValue = null;
						
						if ( objectIsStruct ) {
							// Currently no structures have array fields so do nothing
						}
						else{
					//		propValue = ( (Data)anObject ).getIDataArray( propName );
							propValue = getDataArray( (Data) anObject, propName);
						}
						
						obj.add( propValue );
					} else {
						// Object property is array of primitive type values
						String [] propValue = null;

						if ( objectIsStruct ) {
							// Currently no structures have array fields so do nothing
						}
						else {
							propValue = getPrimitivePropArray( (Data) anObject, def, prop );
						}
						
						obj.add( propValue );
					}
				} else { // End of if ( ObjectData.isSequence( prop ) )
					// Object property contains a single value

					if ( ObjectData.isStruct( prop ) == 1 ) {
						// Object property is a structure
						Object propValue = null;
						
						if ( objectIsStruct ) {
							propValue = getStructField( anObject, def, prop );
						}
						else {
							propValue = getStructNew((Data) anObject, prop.getName());
						//	propValue = getStruct( (Data) anObject, def, prop.getName() );
						}
						
						obj.add( propValue );
					}
					else if ( ObjectData.isObject( prop.getType() ) ) {
						// Object property is another IData object

						Object propValue = null;
						
						if ( prop.getType().equals( "IImage" ) ) {
							propValue = (IImage) getIImage( (Data) anObject, prop.getName());				
						} else if( prop.getType().equals( "ISellable" ) ) {
							propValue = "Sellable";
						}
						else if (objectIsStruct) {
							propValue = getStructField( anObject, def, prop );
						} else
						//	propValue = ( (Data) anObject ).getIDataValue( propName );
							propValue = getDataObject( (Data) anObject, propName);
						
							obj.add( propValue );
					} else {
						// Object property is a primitive value
						String propValue = null; 

						if ( objectIsStruct ) {
							propValue = getStructField( anObject, def, prop ).toString();
						} else {
							// IRegion does not have explicit getters for
							// population and numberofworkers fields
							if ( def.getName().equals("Region") &&
									( ( propName.toLowerCase().equals("population") 
									 || propName.toLowerCase().equals("numberofworkers") ) ) ) {
								Region region = (Region) anObject;
								propValue = "" + region.getValue( propName );							
							} else {
						//		propValue = ( (Data) anObject).getFieldValue( propName );
								propValue = getFieldValueSU((Data) anObject, propName);
							} 
						}

						// Create the civil name if there exist such properties
						// as defined in TOM
						if ( prop.isCivil() ) {
							civilName += ( " " + propValue );
							hasCivilName = true;
						}
						obj.add( propValue );

					}
				}
			} catch ( Exception e ) {
				System.err.println( "Unable to retreive property " + propName 
						+ " of object "	+ def.getName() + ".");
				obj.add( null );
				e.printStackTrace();
			}
		}
		
		ObjectData retrObject = new ObjectData( def, obj, anObject );
		if ( hasCivilName )
			retrObject.setCivilName( civilName );
		return retrObject;
	}
	
	private Object getStructField( Object struct, ObjectDef structDef, PropertyDef propDef ) {
		try {
			Class objectClass = Class.forName( structDef.getClassName() );
			Field f = objectClass.getDeclaredField( propDef.getName() );
			return f.get(struct);
		} catch (ClassNotFoundException e) {
			System.err.println( "Attempt to access non-existing object '" +
					structDef.getName() + "'." );
			return null;
		} catch (IllegalAccessException e) {
			System.err.println( "Invalid attempt to access field '" +
					propDef.getName() + "' of object '" + structDef.getName() + "'." );
			return null;
		} catch (NoSuchFieldException e) {
			System.err.println( "Attempt to access non-existing public field '" +
					propDef.getName() + "' of object '" + structDef.getName() + "'." );
			return null;
		} catch ( NullPointerException e ) {
			System.err.println( "SComm.getStructProperty: unknown error when " +
					"accessing object '" + structDef.getName() + "'." );
			return null;
		}
	}

	private String getOPName(OperationProcess op) {
		if(op != null) {			
			if (op instanceof OPFinance) return "OPFinance";
			if (op instanceof OPLogistics) return "OPLogistics";
			if (op instanceof OPResearch) return "OPResearch";
			if (op instanceof OPManufacturing) return "OPManufacturing";
			if (op instanceof OPPM) return "OPPM";
			if (op instanceof OPPortfolio) return "OPPortfolio";
			if (op instanceof OPProject) return "OPProject";
			if (op instanceof OPSales) return "OPSales";
			
			if (op instanceof OperationProcess) return "OperationProcess";
			return op.getTypeName();
		}
		else return "";
	}

	private String getClauseName(Clause c) {
		if(c != null) {			
			if (c instanceof DurationClause) return "DurationClause";
			if (c instanceof PenaltyClause) return "PenaltyClause";
			if (c instanceof ProductClause) return "ProductClause";
			if (c instanceof Clause) return "Clause";
			return c.getTypeName();
		}
		else return "";
	}

	private String getFUName(FunctionalUnit ifu) {
		if(ifu != null) {
			if (ifu instanceof FinanceUnit)	return "FinanceUnit";
			if (ifu instanceof ProductionUnit) return "ProductionUnit";
			if (ifu instanceof DistributionUnit) return "DistributionUnit";
			if (ifu instanceof DevelopmentUnit) return "DevelopmentUnit";
			if (ifu instanceof SalesUnit) return "SalesUnit";
			return ifu.getTypeName();
		}
		else return "";
	}

	public boolean setPropertyValue( ObjectData objData, PropertyDef propDef, String value ) {
		try {
			Data object = (Data) objData.getServerObjectReference();
			if ( object == null ) {
				System.err.println( "Cannot get " + objData.getCivilName() + " data object." );
				return false;
			}
			
		
			String fieldName = propDef.getName();
			if ( propDef.getType().equals( "date" ) && value.contains( "-" ) )
				value = new Long( string2Date( value ) ).toString();
			boolean retValue = object.setFieldValue( fieldName, value );
			if ( retValue ){
				Session se = dataManager.openSession();
				object.save(se);
				dataManager.closeSession(se);
			}
			else 
				System.out.println( "Unable to set field '" 
						+ fieldName + "' with value '" + value
						+ "' of object '" + objData.getName() + "'.");
			return retValue;
		} catch (NumberFormatException e) {
			System.err.println( "Uncorrect data entered" );
			return false;
		} catch ( ClassCastException e ) {
			System.err.println( objData.getCivilName() + " ObjectData does not contain" +
					" valid IData object reference" );
			return false;
		}
		catch ( Exception e ) {
			System.err.println( "Error occured when trying to set field '" 
					+ propDef.getName() + "' with value '" + value
					+ "' of object '" + objData.getName() + "'.");
			return false;
		}	
	}

	public boolean setPropetyObjectValue( Data mainObject, String mainObjectName,
			Data []value, String valueName ) throws SCommException {

		ObjectDef def = objects.getObjectDef( mainObjectName );
		if( def == null )
			throw new SCommException( "Object " + mainObjectName + " not defined!" );
		
		boolean retvalue = setIDataValue( mainObject, def.getClassName(), valueName, 
				def.getPropertyDef(valueName).getType(), value ); 
		
		Session se = dataManager.openSession();
		mainObject.save(se);
		dataManager.closeSession(se);
		
		return retvalue;
	}
	
	/**
	 * Thе <code>setObject2Server( ArrayList, String )</code> function
	 * should be used by the dataObjects importer. It creates a data 
	 * object on the server and then sets all of its primitive (and only
	 * primitive) properties.
	 * 
	 * @param propValues 	Ordered (according to the current TOM object
	 * 						definition) list of primitive property values.
	 * 
	 * @param objectName	The name of the data object to be created,
	 * 						according to the current TOM object definition).
	 * 
	 * @return				The newly created data object.
	 * @throws SCommException 
	 */
	public Data setObject2Server( ArrayList<String> propValues, String objectName ) throws SCommException {
		ObjectDef def = objects.getObjectDef( objectName );
		if( def == null )
			throw new SCommException( "Object " + objectName + " not defined!" );

		DataObject dataObject = _serverObject.createDataObject( def.getClassName() );
		
		Session se = dataManager.openSession();
		Data newDataObject = Data.fromDataObject(se, dataObject);
		newDataObject.save(se);
		dataManager.closeSession(se);
				
		for ( int i = 0; i < def.size(); i++ ) {
			String value = propValues.get( i );
			String fieldName = def.getPropertyDef( i ).getName();
			
			try {
				if ( value != null) {
					if ( !newDataObject.setFieldValue( fieldName, value ) ) {
						System.out.println( "Unable to set field '"	+ fieldName + 
								"' with value '" + value + "' of object '" + objectName + "'.");
					} else {
						
				//dali da go se otavria vseki pyt session???		
						Session se1 = dataManager.openSession();
						newDataObject.save(se1);
						dataManager.closeSession(se1);
					}
				}
			} catch (RuntimeException e) {
				System.err.println( "Error occured when trying to set field '" 
						+ fieldName + "' with value '" + value
						+ "' of object '" + objectName + "'.");
				e.printStackTrace();
			}
		}
		
		Session se2 = dataManager.openSession();
		newDataObject.save(se2);
		dataManager.closeSession(se2);
		return newDataObject;	
	}
		
	/**
	 * The <code>setObject2Server( ObjectData )</code> function creates a
	 * new data object and sets all of it non-readonly properties.
	 * 
	 * @param anObject	TOM object
	 * @return			Reference to <code>IData</code> object if the operation
	 * 					is successfull and <code>null</code> otherwise.
	 */
	
	public Data setObject2Server( ObjectData anObject ) {
		boolean retValue = false;
		String className = anObject.getClassName();
				
		Data newObject = null;
		
		try {
			newObject = (Data) Class.forName(className).newInstance();
			Session se = dataManager.openSession();
			newObject.save(se);
		//	dataManager.storeDataA(newObject);
			dataManager.closeSession(se);
			
			System.out.println(newObject.getId());
			
		} catch (Exception e) {
			e.printStackTrace();
		}
				
		if ( newObject == null) return null;	
	
		for( int i = 0; i < anObject.size(); i++ ) {
			PropertyDef def = anObject.getPropertyDef( i );
			
			if ( !def.isReadonly() && 
					(anObject.getPropertyIData(i) != null || anObject.getPropertySequData(i) != null) ) {
				
				String fieldName = def.getName();
				String fieldType = def.getType();

				try {
					if (ObjectData.isSequence(def)){
						if (ObjectData.isStruct(def) == 1){
						//  sequence of structures
							
						} else if (ObjectData.isObject(def.getType())){
						//	sequence of objects
							Data[] fieldValueData = { (Data) anObject.getPropertyIData( i ) };
							if (fieldValueData[0] != null){
								retValue = setIDataValue( newObject, className,
									fieldName, fieldType, fieldValueData );
							}
						} else {
						//	sequence of primitive types
							ArrayList aList = anObject.getPropertySequData(i);
							Object[] array = aList.toArray();
							System.out.println("fieldType: " + fieldType);
							setPrimitiveArray(newObject, array, fieldName, fieldType);			
						 
						}
					} else {
						if (ObjectData.isStruct(def) == 1){
						//	single structure
							Object obj = anObject.getPropertyIData(i);
							String strClassName = SComm.getObjects().getObjectDef(fieldType).getClassName();
							Class strClass = Class.forName(strClassName);
							Object[] strInstance = new Object[1]; 
							strInstance[0] = strClass.newInstance();
							strInstance[0] = obj;
							
						//	setStructureFields(newObject, strInstance, fieldName, strClass);
							setStructureFields(newObject, obj, fieldName, strClass);
							
						} else if (ObjectData.isObject(def.getType())){
						//	single object
							
							if(def.getType().equals("IImage")){
			//					Object image = anObject.getPropertyIData(i);
			//					if (image != null){
			//						retValue = setIImage(newObject, helperClassName, className,
			//						 fieldName, image);
			//					}
								continue;
							} 
							Data []fieldValueData = { (Data) anObject.getPropertyIData( i ) };
							if (fieldValueData[0] != null){
								retValue = setIDataValue( newObject, className,
									fieldName, fieldType, fieldValueData );
							}
						} else {
						//	single primitive type	
							if (anObject.getPropertyData( i ) != null){
								retValue = newObject.setFieldValue( fieldName, anObject.getPropertyData( i ) );
							}
						}
					}					
					
				} catch ( ClassCastException e) {
					System.err.println( e.getMessage() );
					System.err.println( "Non Data value at field '"
							+ fieldName + "' of object '" + className + "'." );
				} catch ( Exception e ) {
					System.err.println( "Error occured when trying to set field '" 
							+ fieldName + "' of object '" + className + "'." );
				}
			}
		}
		
		Session se1 = dataManager.openSession();
	//	dataManager.storeData(se1, newObject);
		newObject.save(se1);
		dataManager.closeSession(se1);

		return newObject;		
	}
	
//	private boolean setIImage(IData object, String helperClassName,
//			String objectClassName, String fieldName, Object image){
//		try{
//			Class objectClass = Class.forName( objectClassName );
//			Object o = objectClass.newInstance();
//			
//		
//			Class objectHelperClass = Class.forName( helperClassName );
//			for ( Method m : objectHelperClass.getMethods() ) {
//				if ( m.getName().equals( "narrow" ) ) {
//					Object[] obj = {object};
//					o = m.invoke( null, obj );
//					break;
//				}
//			}
//				
//			String method = "set".concat( fieldName.substring( 0, 1 ).toUpperCase().concat( fieldName.substring(1) ) );		
//			if (image != null){
//				for ( Method m : o.getClass().getMethods() ){
//					if ( m.getName().equals( method ) ){
//						m.invoke( o, image );
//						return true;
//					}
//				}
//			}
//			else{
//				for ( Method m : o.getClass().getMethods() ){
//					if ( m.getName().equals( method ) ){
//						m.invoke( o, new Object[]{null} );
//						return true;
//					}
//				}
//			}		
//			
//			object.save();
//		} catch (ClassNotFoundException e) {
//			e.printStackTrace();
//		} catch (InstantiationException e) {
//			e.printStackTrace();
//		} catch (IllegalAccessException e) {
//			e.printStackTrace();
//		} catch (ClassCastException e) {
//			e.printStackTrace();
//		} catch (Exception e) {
//			e.printStackTrace();
//		}
//
//		return false;
//		
//		
//	}
//
	private boolean setIDataValue( Data object, String objectClassName, 
			String fieldName, String fieldType, Data[] value ) {
		try {
		
			Class objectClass = Class.forName( objectClassName );
			Object o = objectClass.newInstance();
			Class valueClass = null;
			Object[] val = new Object[value.length];
			
			o = object;
			if ( value != null ) {
				val = new Object[value.length];
		
				if ( value[0].getTypeName().equals( fieldType ) ) {
					ObjectDef def = objects.getObjectDef( fieldType );
					if( def == null ) {
						System.err.println( "Fatal error in SComm.SetIDataValue. Operation stopped." );
						return false;
					}
	
					String fieldClassName = def.getClassName();
					
					valueClass = Class.forName( fieldClassName );
					for ( int j = 0; j < value.length; j++ ) {
						val[j] = valueClass.newInstance();
						val[j] = value[j];
					}			
				}
			} 
			else 
				val = new Object[]{null};
			
			String method = "set".concat( fieldName.substring( 0, 1 ).toUpperCase().concat( fieldName.substring(1) ) );		
			
		//	System.out.println("method: " + method + "; fieldName: " + fieldName);
			
			for ( Method m : o.getClass().getMethods() ){
				if ( m.getName().equals( method ) ){
					m.invoke( o, val );
					return true;
				}
			}
			
			Session se = dataManager.openSession();
			object.save(se);
			dataManager.closeSession(se);
			
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (ClassCastException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}

		return false;
	}
	
	private String[] getPrimitivePropArray( Data object, ObjectDef objDef, PropertyDef propDef ) {
		String[] retValue = null;
		Session se = dataManager.openSession();
    	
		try {			
			Class objectClass = Class.forName( objDef.getClassName() );
			Object o = objectClass.newInstance();
		
			o = object;
			
			String methodName = "get".concat( propDef.getName().substring( 0, 1 ).toUpperCase().concat( propDef.getName().substring(1) ) );
			Method method = null;
			for (Method m : o.getClass().getMethods()){
				if (m.getName().equals(methodName)){
					method = m;
					break;
				}
			}
			
			if ( propDef.getType().toLowerCase().equals( "string" ) ) {
				if (method.getParameterTypes().length == 0)
					retValue = (String[])method.invoke( o, new Object[0] );
				else
					retValue = (String[]) method.invoke(o, se);

			} 
			else if ( propDef.getType().toLowerCase().equals( "int" ) ) {
				int[] values = null;
				if (method.getParameterTypes().length == 0)
					values = (int[]) method.invoke(o, new Object[0]);
				else
					values = (int[]) method.invoke(o, se);				
				
				if (values != null){
					retValue = new String[values.length];
					for( int i = 0; i < values.length; i++ )
						retValue[i] = new Integer( values[i] ).toString();
				}else
					retValue = null;
			} 
			else if ( propDef.getType().toLowerCase().equals( "long" )){
				long[] values = null;
				if (method.getParameterTypes().length == 0)
					values = (long[]) method.invoke(o, new Object[0]);
				else
					values = (long[]) method.invoke(o, se);				
				
				if (values != null){
					retValue = new String[values.length];
					for( int i = 0; i < values.length; i++ )
						retValue[i] = new Long( values[i] ).toString();
				}else
					retValue = null;
			} 
			else if (propDef.getType().toLowerCase().equals( "double" )){
				double [] values = null;
				
				if (method.getParameterTypes().length == 0)
					values = (double[]) method.invoke(o, new Object[0]);
				else
					values = (double[]) method.invoke(o, se);	
				
				if (values != null){
					retValue = new String[values.length];
					for( int i = 0; i < values.length; i++ )
						retValue[i] = new Double( values[i] ).toString();
				}else
					retValue = null;
			}			
			else if ( propDef.getType().toLowerCase().equals( "float" )) {
				float [] values = null;
				
				if (method.getParameterTypes().length == 0)
					values = (float[]) method.invoke(o, new Object[0]);
				else
					values = (float[]) method.invoke(o, se);	
				
				if (values != null){
					retValue = new String[values.length];
					for( int i = 0; i < values.length; i++ )
						retValue[i] = new Float( values[i] ).toString();
				}else
					retValue = null;
			} 
			else {								
				retValue = new String[] { "Not supported yet..." } ;
			}
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			System.err.println("Unable to retreive property " + propDef.getName() +
					" of the abstract object (class) " + objDef.getClassName() );
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		} catch (ClassCastException e) {
			e.printStackTrace();
		} finally {
			dataManager.closeSession(se);
		}

		return retValue;
		
	}
	
	private Object getStruct( Data parentObject, ObjectDef parentDef, String structName ) {
		try {
			Class objectClass = Class.forName( parentDef.getClassName() );
			Object o = objectClass.newInstance();
					
	//		String helperClassName = ObjectData.getPackageName( parentDef ).concat( parentDef.getName()).concat("Helper");
	//		Class objectHelperClass = Class.forName( helperClassName );
			
	//		for ( Method m : objectHelperClass.getMethods() ) {
	//			if ( m.getName().equals( "narrow" ) ) {
	//				o = m.invoke( null, new Object[]{parentObject});
	//				break;
	//			}
	//		}
			
			o = parentObject;
			
			String method = "get".concat( structName.substring( 0, 1 ).toUpperCase().concat( structName.substring(1) ) );
//			return new Object[] { o.getClass().getMethod(method, new Class[0]).invoke( o, new Object[0]) };
			return o.getClass().getMethod(method, new Class[0]).invoke( o, new Object[0]);

		} catch ( ClassNotFoundException e ) {
			System.err.println( "Attempt to access non-existing object '" +
					parentDef.getName() + "'." );
			return null;
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		} catch (InvocationTargetException e) {
			System.err.println("Exception in method SComm.getStruct caused by invoking '" + "get".concat(structName) + "' method of object '" + parentDef.getClassName() + "'" );
			// TODO Auto-generated catch block
		//	e.printStackTrace();
			return null;
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		} catch (NoSuchMethodException e) {
			System.err.println( "Attempt to access non-existing property '" +
					structName + "' of object '" + parentDef.getName() + "'." );
			return null;
		}
	}

	
	public boolean setIDataArray( Data object, Data[] params, PropertyDef propertyDef ){

		String retType = propertyDef.getType();  
		
		ObjectDef propObjectDef = SComm.getObjects().getObjectDef(retType);
		
		String arrayType = propertyDef.getType();
		
		String mainClass = propObjectDef.getClassName();
//			mainObjectData.getClassName();	
//		String propStubClass = ObjectData.getPackageName( propObjectDef ).concat("_").concat(retType).concat("Stub");
		
//		String mainClassHelper = mainObjectData.getPackageName().concat(mainObjectData.getName()).concat("Helper");
//		String propHelper = ObjectData.getPackageName( propObjectDef ).concat(retType).concat("Helper");
		
		String methodName = "set".concat(propertyDef.getName());	
	
		try {
			Class objectClass = Class.forName( mainClass ); 
			Data o = (Data) objectClass.newInstance();
			
			o = object;
			
//			Class objectHelperClass = Class.forName( mainClassHelper ); //ISiteHelper
//			for ( Method m : objectHelperClass.getMethods() ) {
//				if ( m.getName().equals( "narrow" ) ) {				
//					o = m.invoke( null, (Object) object );
//					break;
//				}
//			}
			Class paramClass = Class.forName(arrayType);
	
			Object paramsObj = Array.newInstance(paramClass, params.length);	
			
			for (int i=0; i<params.length; i++){
				Array.set(paramsObj, i, paramClass.newInstance());	
				Object param = paramClass.newInstance();
				param = params[i];
				Array.set(paramsObj, i, param);		
			}
			
//			Class paramHelperClass = Class.forName( propHelper );
			
//			for ( Method m : paramHelperClass.getMethods() ) {
//				if ( m.getName().equals( "narrow" ) ) {
//					for ( int j = 0; j<params.length; j++ ){
//						Object param = m.invoke( null, (org.omg.CORBA.Object) params[j] );
//						Array.set(paramsObj, j, param);			
//					}
//					break;
//				}
//			}
//			
			for ( Method m : o.getClass().getMethods() ){
				if ( m.getName().equals( methodName ) ){				
					m.invoke( o, paramsObj );					
				}
			}
			
			Session se = dataManager.openSession();
			o.save(se);
			dataManager.closeSession(se);
			
		}catch (ClassNotFoundException e) {
			e.printStackTrace();			
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (ClassCastException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;	
	}
	
	public boolean setPrimitiveArray(Data object, Object[] params, ObjectData mainObjectData, 
			PropertyDef propertyDef ){		
//				
//		String mainClass = mainObjectData.getClassName();	
//		String mainClassHelper;		
//		
//		String setMethodName = "set".concat(propertyDef.getName());		
//		String getMethodName = "get".concat(propertyDef.getName());		
//		
//		mainClassHelper = mainObjectData.getPackageName().concat(mainObjectData.getName()).concat("Helper");		
//		try {
//			Class objectClass = Class.forName( mainClass ); 
//			Object o = objectClass.newInstance();			
//			Class objectHelperClass = Class.forName( mainClassHelper ); 
//			for ( Method m : objectHelperClass.getMethods() ) {
//				if ( m.getName().equals( "narrow" ) ) {
//					o = m.invoke( null, (Object) object );
//					break;
//				}
//			}		
//			
//			String strClass = params[0].getClass().getCanonicalName();
//			Class paramClass = Class.forName(strClass);			
//		
//			Object paramsObj = Array.newInstance(paramClass, params.length);			
//			
//			for (int i=0; i<params.length; i++){
//				Array.set(paramsObj, i, paramClass.newInstance());		
//			}			
//		
//			for ( int j = 0; j<params.length; j++ ){
//				Array.set(paramsObj, j, params[j]);
//			}		
//			
//			for ( Method m : o.getClass().getMethods() ){
//				if ( m.getName().equals( setMethodName ) ){
//					m.invoke( o, paramsObj );		
//					object.save();					
//					break;			
//				}
//			}
//			
//			for ( Method m : o.getClass().getMethods() ){
//				if ( m.getName().equals( getMethodName ) ){
//					Object[] val = (Object[]) m.invoke(o, new Object[] {});
//					int i = 0;
//					for (Object obj : val){
//						System.out.println("Object " + i + ": " + obj.toString());
//						i++;
//					}
//					break;
//				}
//			}
//			return true;
//			
//			
//		}catch (ClassNotFoundException e) {
//			e.printStackTrace();			
//		} catch (InstantiationException e) {
//			e.printStackTrace();
//		} catch (IllegalAccessException e) {
//			e.printStackTrace();
//		} catch (ClassCastException e) {
//			e.printStackTrace();
//		} catch (Exception e) {
//			e.printStackTrace();
//		}
		return false;	
	}
//	
	private boolean setPointValue( Data object, String objectClassName, String fieldName, prime.net.Point p ) {
		try {
		
			Class objectClass = Class.forName( objectClassName );
			Object o = objectClass.newInstance();
			o = object;				
		
			String method = "set".concat( fieldName.substring( 0, 1 ).toUpperCase().concat( fieldName.substring(1) ) );		
			for ( Method m : objectClass.getMethods() ){
				if ( m.getName().equals( method ) ){
					m.invoke( o, p );
					Session se = dataManager.openSession();
					object.save(se);
					dataManager.closeSession(se);
					return true;
				}
			}			
			
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (ClassCastException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}

		return false;
	}
	
	private boolean setPointArrayValue( Data object, String objectClassName, String fieldName, prime.net.Point[] p ) {
		try {			
			Class objectClass = Class.forName( objectClassName );
			Object o = objectClass.newInstance();
			o = object;	
		//	if ( p != null ) {
		
		//		Class objectHelperClass = Class.forName( helperClassName );
		//		for ( Method m : objectHelperClass.getMethods() ) {
		//			if ( m.getName().equals( "narrow" ) ) {
		//				Object[] obj = {object};
		//				o = m.invoke( null, obj );
		//				break;
		//			}
		//		}
		//	}
			
	//		Class paramClass = Class.forName(prime.net.Point.class.getCanonicalName());			
			
	//		Object paramsObj = Array.newInstance(paramClass, p.length);			
			
	//		for (int i=0; i<p.length; i++){
	//			Array.set(paramsObj, i, paramClass.newInstance());		
	//		}			
		
	//		for ( int j = 0; j<p.length; j++ ){
	//			Array.set(paramsObj, j, p[j]);
	//		}
			
			String method = "set".concat( fieldName.substring( 0, 1 ).toUpperCase().concat( fieldName.substring(1) ) );		
			for ( Method m : objectClass.getMethods() ){
				if ( m.getName().equals( method ) ){
					m.invoke( o,(Object) p );
					Session se = dataManager.openSession();
					object.save(se);
					dataManager.closeSession(se);
					return true;
				}
			}
			
			
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (ClassCastException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}

		return false;
	}
	
	private Data[] getDataArray(Data object, String fieldName){
		Session se = dataManager.openSession();
		
		try {
			Class cl = Class.forName("org.hibernate.Session");
			String method = "get".concat(fieldName.substring(0,1).toUpperCase().concat(fieldName.substring(1)));
			Object o = object.getClass().getMethod(method, cl).invoke(object, se);
			if (o instanceof List) return ((List<Data>)o).toArray( new Data[0]);
			return null;
		} catch (Exception e) {
			return null;
		} finally {
			dataManager.closeSession(se);
		}
	}
	
	private Data getDataObject(Data object, String fieldName){
		Session se = dataManager.openSession();
		
		try {
			Class cl = Class.forName("org.hibernate.Session");
			String methodName = "get".concat(fieldName.substring(0,1).toUpperCase().concat(fieldName.substring(1)));
			Method method = null;
			for (Method m : object.getClass().getMethods()){
				if (m.getName().equals(methodName)){
					method = m;
					break;
				}
			}
			
			Object o = null;
			if (method.getParameterTypes().length == 0){
				o = method.invoke(object, new Object[0]);
			}else {
				o = method.invoke(object, se);
			}			
		
			if (o instanceof Data) return (Data)o;
			return null;
			
		} catch (Exception e) {
			return null;
		} finally {
			dataManager.closeSession(se);
		}
	}
	
	private IImage getIImage(Data object, String fieldName){
		String methodName = "get".concat(fieldName.substring(0,1).toUpperCase().concat(fieldName.substring(1)));
		Session se = dataManager.openSession();
		
		try{
			Method method = null;
			for (Method m : object.getClass().getMethods()){
				if (m.getName().equals(methodName)){
					method = m;
					break;
				}
			}		
			
			Object o = null;
			if (method.getParameterTypes().length == 0){
				o = method.invoke(object, new Object[0]);
			} else {
				o = method.invoke(object, se);
			}
			
			IImage image = IImageHelper.narrow( (org.omg.CORBA.Object) o);
			return image;
			//	return (IImage) o;
			
		}catch (Exception e){
		//	e.printStackTrace();
			System.err.println("Cannot get Image of object " + object.getClass().getSimpleName());
			return null;
		} finally {
			dataManager.closeSession(se);
		}
	}
	
	private Object getStructNew( Data object, String structName ) {
		Session se = dataManager.openSession();
		
		try {			
			String methodName = "get".concat(structName.substring(0,1).toUpperCase().concat(structName.substring(1)));
			Method method = null;
			for (Method m : object.getClass().getMethods()){
				if (m.getName().equals(methodName)){
					method = m;
					break;
				}
			}			
			
			Object o = null;
			if (method.getParameterTypes().length == 0){
				o = method.invoke(object, new Object[0]);
			}else {
				o = method.invoke(object, se);
			}			
		//	System.out.println(o.getClass().getSimpleName());
			return o;		
			
		} catch (Exception e) {
			return null;
		} finally {
			dataManager.closeSession(se);
		}		
	}
	
	private String getFieldValueSU(Data object, String fieldName){
		Session se = dataManager.openSession();
		
		try {			
			String methodName = "get".concat(fieldName.substring(0,1).toUpperCase().concat(fieldName.substring(1)));
			Method method = null;
			for (Method m : object.getClass().getMethods()){
				if (m.getName().equals(methodName)){
					method = m;
					break;
				}
			}			
			
			Object o = null;
			if (method.getParameterTypes().length == 0){
				o = method.invoke(object, new Object[0]);
			}else {
				o = method.invoke(object, se);
			}			
		//	System.out.println(o.getClass().getSimpleName());
			return o.toString();		
			
		} catch (Exception e) {
			return null;
		} finally {
			dataManager.closeSession(se);
		}		
	}	
	
	public void setStructureFields(Data object, Object structure, String fieldName, Class strClass){
		String methodName = "set".concat(fieldName);
		System.out.println("methodName " + methodName);
	
		try{			
			Method method = object.getClass().getMethod(methodName, strClass);
			method.invoke(object, structure);
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
	public void setPrimitiveArray(Data object, Object[] array, String fieldName, String fieldType){
		String methodName = "set".concat(fieldName);	
		if (fieldType.equals("int")){
			int[] newArray = new int[array.length];		
			for (int i = 0; i < array.length; i++){
				newArray[i] = Integer.parseInt(array[i].toString());			
			}	
			try{			
				System.out.println("*****   " + newArray.getClass().getCanonicalName() + " method name: " + methodName + ": fieldType: " + fieldType);
				Method method = object.getClass().getMethod(methodName, newArray.getClass());
				method.invoke(object, newArray);
			} catch (Exception e){
				e.printStackTrace();
			}
		}
		else if (fieldType.equals("long")){
			long[] newArray = new long[array.length];		
			for (int i = 0; i < array.length; i++){
				newArray[i] = Long.parseLong(array[i].toString());			
			}	
			try{			
				System.out.println("*****   " + newArray.getClass().getCanonicalName() + " method name: " + methodName + ": fieldType: " + fieldType);
				Method method = object.getClass().getMethod(methodName, newArray.getClass());
				method.invoke(object, newArray);
			} catch (Exception e){
				e.printStackTrace();
			}
		}
		else if (fieldType.equals("double")){
			double[] newArray = new double[array.length];		
			for (int i = 0; i < array.length; i++){
				newArray[i] = Double.parseDouble(array[i].toString());			
			}	
			try{			
				System.out.println("*****   " + newArray.getClass().getCanonicalName() + " method name: " + methodName + ": fieldType: " + fieldType);
				Method method = object.getClass().getMethod(methodName, newArray.getClass());
				method.invoke(object, newArray);
			} catch (Exception e){
				e.printStackTrace();
			}
		}
		else if (fieldType.equals("float")){
			float[] newArray = new float[array.length];		
			for (int i = 0; i < array.length; i++){
				newArray[i] = Float.parseFloat(array[i].toString());			
			}
			try{			
				System.out.println("*****   " + newArray.getClass().getCanonicalName() + " method name: " + methodName + ": fieldType: " + fieldType);
				Method method = object.getClass().getMethod(methodName, newArray.getClass());
				method.invoke(object, newArray);
			} catch (Exception e){
				e.printStackTrace();
			}
		}
		else{ //fieldType = string
			String[] newArray = new String[array.length];		
			for (int i = 0; i < array.length; i++){
				newArray[i] = array[i].toString();			
			}
			try{			
				System.out.println("*****   " + newArray.getClass().getCanonicalName() + " method name: " + methodName + ": fieldType: " + fieldType);
				Method method = object.getClass().getMethod(methodName, newArray.getClass());
				method.invoke(object, (Object) newArray);
			} catch (Exception e){
				e.printStackTrace();
			}
		}		
	}
	
	private String getType(String fieldType){
		if (fieldType.equals("string"))
			return "java.lang.String";
		if (fieldType.equals("int"))
			return "java.lang.Integer";
		else return ""	;
	}
	
}