package prime.toolbox.test;

import java.util.ArrayList;

import prime.toolbox.ObjectData;
import prime.toolbox.scomm.PseudoSComm;
import prime.toolbox.scomm.SCommException;
import prime.toolbox.xml.reader.ReaderException;

/*
 * This is a test example. It does this:
 * 
 * 1. creates a pseudo comm object, loads TOM.XML file 
 * 2. browses manually the simulated VBE database by
 * 		showing the top-level VBE object, the umber of regions
 * 		and lists all players names along with the number of
 * 		their business units and their names.
 * 3. browses automatically the simulated VBE at depth level 5.
 * 		I.e. the VBE is unfolded so that the first 5 levels of
 * 		object hierarchy is displayed. Note: slef-recursive
 * 		objects like REGION are unfolded only one level. 
 * 
 * @author      Pavel Boytchev
 * @version     1.2, 2006-08-28
 */
public class TestPseudoSComm {
	private static void dump(PseudoSComm comm, ObjectData object, String prefix, int level) throws SCommException
	{
		if (level==0) return;
		if (object.getCivilName()==null)
			System.out.println( prefix+"*"+object.stringOf());
		else
			System.out.println( prefix+object.getName()+" "+object.getCivilName());
				
		// scan for local subobjects
		for (int i=0; i<object.size(); i++)
		{
			if (object.isObject(i) && !object.getPropertyDef(i).getType().equals(object.getName()))
				if (object.isSequence(i))
				{	// the subobject is a sequence of objects
					ArrayList subobjects=object.getPropertySeqData(i);
					for(int j=0; j<subobjects.size(); j++)
					{
						ObjectData subobject = comm.retrieveObject((String)subobjects.get(j));
						dump(comm,subobject,prefix+"     ",level-1);
					}
				}
				else
				{	// the subobject is a single object
					ObjectData subobject = comm.retrieveObject(object.getPropertyData(i));
					dump(comm,subobject,prefix+"     ",level-1);
				}
		}
	}
	
	public static void main(String[] args) {
		try
		{
			PseudoSComm comm = new PseudoSComm("prime/toolbox/test/tom.xml");
			
			
			ObjectData vbe = comm.retrieveObject("VBE",1);
			
			System.out.println("-------------------");
			System.out.println("MANUAL VBE BROWSING");
			System.out.println("-------------------");
			System.out.println("Top-level: "+vbe.stringOf());
			System.out.println("Number of regions: "+vbe.getPropertySeqData(1).size());
			System.out.println("Players:");			
			ArrayList players = vbe.getPropertySeqData(2);
			for( int i=0; i<players.size(); i++)
			{
				// get the name of a player
				ObjectData player = comm.retrieveObject((String)players.get(i));
				System.out.print("   "+(i+1)+". "+player.getPropertyData(0));
				// get the number of companies
				ArrayList companies = player.getPropertySeqData(1);
				System.out.print(" has "+companies.size()+" companies: ");
				for ( int j=0; j<companies.size(); j++ )
				{
					ObjectData company = comm.retrieveObject((String)companies.get(j));
					System.out.print(company.getCivilName()+" ");
				}
				System.out.println();
			}
		
			
			System.out.println("\n");
			System.out.println("--------------------------------");
			System.out.println("AUTOMATIC VBE BROWSING - LEVEL 5");
			System.out.println("--------------------------------");
			dump(comm,vbe,"",5);
		}
		catch(ReaderException txre)
		{
			System.err.println( "Reader: "+txre.getMessage() );
		}
		catch(SCommException sce)
		{
			System.err.println( "SCom: "+sce.getMessage() );
		}
	}

}
