package prime.toolbox.xml.exporter;

import java.io.*;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import org.hibernate.Session;
import prime.game.dataobjects.*;
import prime.game.dataobjects.production.*;
import prime.toolbox.*;
import prime.toolbox.scomm.SComm;
import prime.toolbox.scomm.SCommException;
import prime.net.Skill;
import prime.client.ClientDataManager;
import java.lang.reflect.Field;

public class Exporter{
	
	/**
	 * <code>CreateXml</code> class is used to create
	 * a <code>xml</code> file that saves all objects.
	 * 
	 * @author      Stanislav Stoykov
	 * @version     2.0, 2007-10-19
	 */
	
	public static void saveXml(String exporterFileName) throws IOException{
		FileWriter outFile = new FileWriter(exporterFileName);
		PrintWriter out = new PrintWriter(outFile);
		
		SComm comm = prime.toolbox.gui.ToolboxObjectViewer.getSComm(); 
		
		ObjectData objData = null;
		
		ObjectCollection objCollect = SComm.getObjects();
		
		String[] names = new String[objCollect.size()];
		
		String separate = "#0#";
		String innerSep = "@00@";
				
		for( int objIdx=0; objIdx<objCollect.size(); objIdx++)	{
			ObjectDef object = objCollect.getObjectDef(objIdx);
			names[objIdx] = object.getName();		
		}	
		
		String add;
		String propertyName;
		String propertyType;
		PropertyDef propertyDef;
		
		Date now = new Date();
		Format formatter = new SimpleDateFormat("MM-dd-yyyy");
			
		add = "<objectdata version=\"1.0\" created=\"" + formatter.format(now) + "\" dateformat=\"MM-dd-yyyy\">";
		out.println(add);
		out.println();
		
		int count = 0;
		for (String type : names){
					
// 		not Data objects
			if (type.equals("IImage"))
				continue;			
			if (type.equals("ISellable"))
				continue;
//  	structures, the structures are not saved in the exporter 	
			if (type.equals("MachineNeed"))
				continue;
			if (type.equals("KPI"))
				continue;
			if (type.equals("Skill"))
				continue;
			if (type.equals("Point"))
				continue;
// 		abstract classes
			if (type.equals("FunctionalUnit"))
				continue;
			if (type.equals("OperationProcess"))
				continue;	
			if (type.equals("Contract"))
				continue;	
			if (type.equals("Clause"))
				continue;	
			
			count++;
			System.out.println(count + " " + type);
			
			Data[] objects = comm.getObjectList(type);
						
			if ( objects != null ) {
				for (int i = 0; i < objects.length; i++) {

					try {
						objData = comm.retrieveObject(objects[i], objects[i].getClass().getSimpleName());
					} catch (SCommException e) {
						System.out.println(e.getMessage());
						e.printStackTrace();
					}
					System.out.println("Object name: " + type
							+ idNumber(objects[i].getId()));
					add = "<object name=\"" + objData.getName()
							+ idNumber(objects[i].getId()) + "\" type=\""
							+ objData.getName() + "\">";
					out.println(add);

					int j = 0;
					while (objData.getPropertyDef(j) != null) {
						System.out.println("      " + j + ": " + objData.getPropertyDef(j).getName());

						propertyDef = objData.getPropertyDef(j);

						propertyName = propertyDef.getName();
						propertyType = propertyDef.getType();

						if (objData.isSequence(j)) {
							ArrayList<Object> list = objData.getPropertySequData(j);
							if (propertyName != null && list != null && list.size() > 0) {

								add = "         <" + propertyName + ">";
								for (int objNum = 0; objNum < list.size(); objNum++) {
									Object object = list.get(objNum);
																
									if (object instanceof Data) {
										Data obj = (Data) object;
										int id = obj.getId();
										if (objNum == 0)
											add += "" + obj.getClass().getSimpleName()
													+ idNumber(id);
										else
											add += separate + obj.getClass().getSimpleName()
													+ idNumber(id);
									} 
									else if (ObjectData.isStruct(propertyDef) == 1) {
										ObjectDef objDef = objCollect.getObjectDef(propertyType);
										String className = objDef.getClassName();
										if (objNum != 0)
											add += separate;
										try{
											Class propetyClass = Class.forName(className);
											Field[] fields = propetyClass.getFields();
											Object o = propetyClass.newInstance();
											o = object;
											for (int fieldsNum = 0; fieldsNum < fields.length; fieldsNum ++){
												if (fieldsNum != 0)
													add += innerSep;
												String fieldType = fields[fieldsNum].getType().getSimpleName().toLowerCase();
												if (ObjectData.isObject(fieldType)){
													Data fieldObject = (Data) fields[fieldsNum].get(o);
													add += fields[fieldsNum].getType().getSimpleName() + idNumber(fieldObject.getId());
												} else{
													add += fields[fieldsNum].get(o).toString();
												}												
											}
										} catch(Exception e){
											e.printStackTrace();
										}
									}									
									else {
										if (objNum == 0)
											add += "" + object.toString();
										else
											add += separate + object.toString();
									}
								}
								add += "</" + propertyName + ">";
								out.println(add);
							}
						} else if (objData.isObject(j) && !objData.isSequence(j)) {

							Object propertyObject = objData.getPropertyIData(j);

							if (propertyType.equals("ISellable") && propertyObject != null) {
								System.out.println("ISellable");
								
								if (propertyObject instanceof Site) {									
									outputRef(propertyName, "Site",	(Data) propertyObject, out);
								} else if (propertyObject instanceof Building) {
									outputRef(propertyName, "Building",(Data) propertyObject, out);
								}
								
							} else if (propertyName != null
									&& propertyObject != null
									&& !propertyType.equals("IImage")) {
								
								if ( ObjectData.isStruct(propertyDef) != 1 ) {
									Data obj = (Data) propertyObject;
									outputRef(propertyName, obj.getClass().getSimpleName(), obj, out);
								}
								else if (ObjectData.isStruct(propertyDef) == 1) {
									ObjectDef objDef = objCollect.getObjectDef(propertyType);
									String className = objDef.getClassName();
									add = "         <" + propertyName + ">";
									try{
										Class propetyClass = Class.forName(className);
										Field[] fields = propetyClass.getFields();
										Object o = propetyClass.newInstance();
										o = propertyObject;
										for (int fieldsNum = 0; fieldsNum < fields.length; fieldsNum ++){
											if (fieldsNum != 0)
												add += innerSep;
											String fieldType = fields[fieldsNum].getType().getSimpleName().toLowerCase();
											if (ObjectData.isObject(fieldType)){
												Data fieldObject = (Data) fields[fieldsNum].get(o);
												add += fields[fieldsNum].getType().getSimpleName() + idNumber(fieldObject.getId());
											} else{
												add += fields[fieldsNum].get(o).toString();
											}											
										}
									} catch(Exception e){
										e.printStackTrace();
									}
									add += "</" + propertyName + ">";
									out.println(add);
								}					
							}
						}

						else {							
							Object propertyData = objData.getPropertyIData(j);

							if (propertyName != null && propertyData != null
									&& propertyType.equals("date")) {
								long x = Long
										.parseLong(propertyData.toString());
								if (x != -1){
									Date d = new Date(x);
									add = "         <" + propertyName + ">"
										+ formatter.format(d) + "</"
										+ propertyName + ">";
									out.println(add);
								}
							} 				

							else if (propertyName != null && propertyData != null) {
								add = "         <" + propertyName + ">"
										+ propertyData.toString() + "</"
										+ propertyName + ">";
								out.println(add);
							}
						}

						j++;

					}
					add = "</object>";
					out.println(add);
					out.println();
					out.flush();
				}
			}			
		}
		out.println("</objectdata>");
		out.flush();
		out.close();
		System.out.println("saveXML() finished.");
		
	}
	
	/**
	 * Crete a number in appropriate format 
	 * which will be used to set an unique name of the object.
	 * 
	 * @return 		The creted number as a String type.
	 */
	public static String idNumber(int n){
		if (n<10) 
			return "000" + n;
		else if (n<100)
			return "00" + n;
		else if (n<1000)
			return "0" + n;
		else return "" + n;					
	}	
	
	/**
	 * Write a line in the data file consisting of name of the property and its value.
	 */
	private static void outputRef(String key, String type, Data obj, PrintWriter out){
		int id = obj.getId();
		String add;
		if (obj != null){
			add = "         <" + key + ">" + type + idNumber(id) + "</" + key + ">" ;
			out.println(add);
		}		
	}
	
	

}
