package prime.toolbox.xml.importer;

import java.io.*;
import java.util.ArrayList;
import java.util.Enumeration;

//import prime.net.dataobjects.IData;
import prime.game.dataobjects.Data;
import prime.toolbox.ObjectCollection;
import prime.toolbox.ObjectDef;
import prime.toolbox.ObjectData;
import prime.toolbox.PropertyDef;
import prime.toolbox.scomm.SComm;
import prime.toolbox.scomm.SCommException;
import prime.toolbox.xml.reader.ReaderException;
import sjg.xml.*;

import java.lang.reflect.Array;
import java.lang.reflect.Field;
import org.hibernate.Session;
import java.util.Hashtable;
import prime.toolbox.xml.exporter.Exporter;
import java.lang.reflect.Method;

/**
 * <code>Importer</code> class reads the object data
 * from an XML file and loads them into the PRIME server.
 *  
 * @author      Pavel Boytchev and Stanislav Stoykov
 * @version     2.0, 2007-11-03
 */
public class Importer {

	private String dateFormat;
	private String version;

	private String fieldName;
	private String fieldValue;
	private String fieldType;

	private String objName;
	private String objType;
	
	private ArrayList<String>	objVarNames    = new ArrayList<String>();
	private ArrayList<Data>		objVarValues   = new ArrayList<Data>();
//	private ArrayList<Boolean>	objVarHasRefs  = new ArrayList<Boolean>();
	private ArrayList<ObjectDef>objVarObjDefs  = new ArrayList<ObjectDef>();
	private ArrayList<Element>	objVarXmlObj   = new ArrayList<Element>();
	PrintWriter logFile;
	
	private String separate = "#0#";
	private String innerSep = "@00@";
	private Hashtable table = new Hashtable();

	private String getData( Element tag, String name) {
		if( tag==null )
			return null;
		
		// try with attribute
    	if ( tag.hasAttribute(name))
			return (tag.getAttribute(name).getValue());
    	
		// try with element
		if ( tag.elements(name).hasMoreElements())
			return (((Element)tag.elements(name).nextElement()).getContents().toString().trim());
		
		// return the default value
		return (null);
    }

	/**
     * Retrieves the version of the XML file.
     * If version attribute does not exist then return "0.0"
     *
     * @return	the version
     */
	public String getVersion() {
		if (version==null)
			return "0.0";
		else
			return version;
	}

	
	private Data findVarValue(String varName)
	{
		int idx;
		for( idx=0; idx<objVarNames.size(); idx++ )
			if( objVarNames.get(idx).equals(varName) )
				return objVarValues.get(idx);
		return null;
	}

	private String getPropValue(Element xmlObject, String propName)
	{
		Enumeration xmlProperties = xmlObject.elements(propName);
		if( xmlProperties.hasMoreElements() )
		{
			Object o = xmlProperties.nextElement();
			if (o instanceof Element)
			{	
				Element xmlProperty = (Element)o;
				return getData(xmlObject,xmlProperty.getName()); // field's value
			}	
		}
		return null;
	}

	private boolean isPrimitive( String type )
	{
		return ( type.equals("string") ||
				 type.equals("float") ||
				 type.equals("boolean") ||
				 type.equals("long") ||
				 type.equals("date")
			);	
	}
	
	public Importer (String tomFilename, String dataFilename ) throws ReaderException, ImporterException, SCommException, IOException {
		try
		{
			SComm scomm = prime.toolbox.gui.ToolboxObjectViewer.getSComm();
			ObjectCollection objects = SComm.getObjects();
			
			File file = new File(dataFilename);
			String logFilename = file.getParent() + "\\importer-log.xml";
			logFile = new PrintWriter( new FileWriter(logFilename) );

			FileInputStream fstream = new FileInputStream(dataFilename);

			Document xmlDocument = Parser.parse(fstream); 
			Element xmlObjectData = xmlDocument.getRoot();
			
			dateFormat = getData(xmlObjectData,"dateformat");

			logFile.println("Importer");
			logFile.println("   TOM file: " + tomFilename);
			logFile.println("  data file: " + dataFilename + " (ver " + getData(xmlObjectData,"version") + "/" + getData(xmlObjectData, "created") + ")");
			logFile.println("   log file: " + logFilename);
			
			//-------------------------------------------------------------
			//  P A S S   1
			//-------------------------------------------------------------
			// Creates all objects 
			//-------------------------------------------------------------
			System.out.println("Importer pass 1");
			logFile.println();
			logFile.println("--------------------------------------");
			logFile.println(" PASS 1 - Creation of all new objects");
			logFile.println("--------------------------------------");
			if (!xmlObjectData.getName().equals("objectdata"))
				throw new ImporterException("Main tag is not <objectdata>");

			int cnt = 0;
			for (Enumeration xmlObjects = xmlObjectData.elements("object"); xmlObjects.hasMoreElements(); )
			{
				cnt++;
				if( cnt % 32 == 0) System.out.print(".");
				Element xmlObject = (Element)xmlObjects.nextElement();
				objType = getData(xmlObject,"type"); // system type of the object
				objName = getData(xmlObject,"name"); // system name of the object
				
				// ======================================================
				// Remove this block when the following objects classes have their
				// default constructor implemented:
				// 1.	DurationClause
				// 2.	PenaltyClause
				// 3.	ProductClause
				// 4.	ProjectClause
				// 5.	Report
				// 6.	SalesReport
				if ( ( objType.contains( "Clause" ) ) || ( objType.contains( "Report" ) ) ) {
					logFile.println("!! Object " + objName + " skipped, currently not possible to be stored on server.");
					continue;
				}
				// =======================================================
					
				if (!objects.hasObjectDef(objType))
				{
					logFile.println();
					logFile.println("! ignoring unknown object of type " + objType);
					continue;
				}
				ObjectDef objectDef = objects.getObjectDef(objType);
					
				logFile.println();
				if( objName == null )
					logFile.println("  importing an instance of " + objType);
				else
					logFile.println("  importing " + objName);
						
				
				Session se = scomm.getDataManager().openSession();
				
				try
				{
					Data newObject;
					if (objectDef.getName().equals("VBE")){
						newObject = SComm.get_vbe();
					} 
					else{
						String className = objectDef.getClassName();
						newObject = (Data) Class.forName(className).newInstance();
						newObject.save(se);
					}
					
					System.out.println(objName + " " + newObject.getId());
					String superClassName = newObject.getClass().getSuperclass().getSimpleName();
					
					table.put(objName, new Integer(newObject.getId()));
					if (!superClassName.equals("Data"))					
						table.put(getBasicName(newObject, objName), new Integer(newObject.getId()));
					
					objVarNames.add(objType + "." + objName);
					objVarValues.add(newObject);
			//		objVarHasRefs.add(hasReferences);
					objVarObjDefs.add(objectDef);
					objVarXmlObj.add(xmlObject);
					logFile.println("  Object " + objName + " written to the server.");
				}
				catch (Exception e)
				{
					logFile.println("! Object " + objName + " failed to be written to the server");
				} finally{
					scomm.getDataManager().closeSession(se);
				}
				
			} //for object enumeration of Pass 1
			System.out.println( "done!" );
			
			//------------------------------------------------------------
			//  P A S S   2
			//------------------------------------------------------------
			// Rescans objects and sets all properties
			//------------------------------------------------------------
			System.out.println("\n\nImporter pass 2\n\n");
			logFile.println();
			logFile.println("--------------------------------------");
			logFile.println(" PASS 2 - Setting all object references");
			logFile.println("--------------------------------------");
			logFile.flush();
			
			// scan all stored objects
			int idx;
			for( idx=0, cnt=0; idx<objVarNames.size(); idx++)
			{
				System.out.println("objVarNames: "  + objVarNames.get(idx));
				
				cnt++;
				if( cnt % 32 == 0) System.out.print(".");
				
	

			// this object has references so try to fix them
				logFile.println("  fixing references in " + objVarNames.get(idx));
				ObjectDef objectDef = objVarObjDefs.get(idx);
				Element xmlObject = objVarXmlObj.get(idx);
				Data sourceObj = objVarValues.get(idx);
				
				// scan object properties
				for( int index=0; index<objectDef.size(); index++)
				{						
					PropertyDef property = objectDef.getPropertyDef(index);
					String propType = property.getType();
					
					// ======================================================
					// Remove this block when the following objects classes have their
					// default constructor implemented:
					// 1.	DurationClause
					// 2.	PenaltyClause
					// 3.	ProductClause
					// 4.	ProjectClause
					// 5.	Report
					// 6.	SalesReport
					if ( ( propType.contains( "Clause" ) ) || ( propType.contains( "Report" ) ) ) {
						logFile.println("!! property " + property.getName() + " skipped, currently not possible to be stored on server.");
						continue;
					}
					// =======================================================

					
					// skip readonly
					if( property.isReadonly()  ) 
						continue;
					
					if (getPropValue( xmlObject, property.getName()) == null)
						continue;
					
					if (ObjectData.isSequence(property)){
						if (ObjectData.isStruct(property) == 1){
							// sequence of structures
							setStructSequence(property, xmlObject, sourceObj,
									objectDef, scomm);
							
						} else if (ObjectData.isObject(property.getType())){
							//sequence of objects
							setObjectSequence(property, xmlObject, sourceObj,
									objectDef, scomm);
						}
						else{								
						//  sequence of primitive types								
							String value = getPropValue( xmlObject, property.getName() );
							if (value != null){
								String[] array = value.split(separate);
								Object[] objectArray = new Object[array.length];
								for (int i = 0; i < array.length; i++)
									objectArray[i] = (Object) array[i];
								
								scomm.setPrimitiveArray(sourceObj, objectArray, property.getName(), property.getType());				
												
							}
						}
					} else{
							if (ObjectData.isStruct(property) == 1){
							// a single structure
							setStructValue(property, xmlObject, sourceObj,
									objectDef, scomm);
						} else if (ObjectData.isObject(property.getType())){
							//a single object
							setObjectValue(property, xmlObject, sourceObj,
									objectDef, scomm);	
							
						} else {
							// a primitive type
							if (property.hasReference()){
								System.out.println("A reference property of object " + objectDef.getName() + 
										". Property name: " + property.getName());
								
								
								setReference(property, xmlObject, sourceObj,
										objectDef, scomm);	
								
							} else {
//								single primitive type	
								String value = getPropValue( xmlObject, property.getName() );
								sourceObj.setFieldValue( property.getName(), value );
																	
								if( value != null)
									logFile.println("     " + property.getName() + " = " + value);
								else
									logFile.println("!    " + property.getName() + " = (undefined (equal to null))");
							}
						}
					}			
				} // for property index

				logFile.println();
					
				Session se = scomm.getDataManager().openSession();
				try {						
					sourceObj.save(se);
				} catch (Exception e){
					e.printStackTrace();
				} finally {
					scomm.getDataManager().closeSession(se);
				}				
					
				logFile.flush();				
				
			} // for objVarNames.size()
			
		System.out.println( "done!" );
		} //main try of Importer()		
		
		
		catch(FileNotFoundException fnfe)
		{
			throw new ImporterException("File not found");
		}		
		logFile.close();
	}
	
	private void setObjectValue(PropertyDef property, Element xmlObject, Data sourceObj,
			ObjectDef objectDef, SComm scomm){
		
		String sourceField = property.getName();
		String value = getPropValue( xmlObject, property.getName() );
	//	System.out.println("----- " + value + " prop: " + property.getName() );
		if( value != null)
		{
			value = property.getType() + "." + value;
			Data targetObj[] = new Data[1];
			targetObj[0] =  findVarValue(value);
			
			if( targetObj[0] != null )
			{
				// now we have these data ready:
				//	  sourceObj   - the Data object containing the link
				//	  sourceField - the name of the field (in sourceObj) containing the link
				//	  targetObj[] - the Data objects pointed to by the link
				try
				{
					if (sourceObj == null)
						System.out.println("sourceObj == null; " + objectDef.getName());
					else
						System.out.println(objectDef.getName());
					
					scomm.setPropetyObjectValue( sourceObj, objectDef.getName(),
							targetObj, property.getName() );
				
					logFile.println("     " + sourceField + " = " + value);
					logFile.println("     --> setPropertyObjectValue($PARENT," + objectDef.getName() + ",$CHILD," + property.getType() + ")");
				}
				catch (SCommException sce)
				{
					logFile.println("!    " + sourceField + " = (failed)");
				} catch (RuntimeException e) {
					System.err.println( "Importer: cannot set " + property.getName() + 
							" of object " + objectDef.getName() );
				}
			}
			else
			{
				logFile.println("!    " + sourceField + " = (cannot find " + value + ")");
			}
		}
		else
			logFile.println("!    " + sourceField + " = (undefined)");
	}
	
	private void setStructValue(PropertyDef property, Element xmlObject, Data sourceObj,
			ObjectDef objectDef, SComm scomm){
		
		String sourceField = property.getName();
		String value = getPropValue( xmlObject, property.getName() );
	//	System.out.println("----- " + value + " prop: " + property.getName() );
		if( value != null)
		{
			String[] array = value.split(innerSep);
			ObjectDef strDef = SComm.getObjects().getObjectDef(property.getType());
			String strName = strDef.getClassName();
			try{
				Class strClass = Class.forName(strName);
				Object str = strClass.newInstance();
				Field[] fields = strClass.getDeclaredFields();
				for (int i = 0; i < fields.length; i++){
					String fieldType = fields[i].getType().getSimpleName();
				//	System.out.println("****: " + fieldType);
					if (ObjectData.isObject(fieldType)){
						Data targetObj =  findVarValue(array[i]);
						if (targetObj != null)
							fields[i].set(str, targetObj);
					}
					else if (fieldType.equals("int")){
						fields[i].setInt(str, Integer.parseInt( array[i]));
					}
					else if (fieldType.equals("long")){
						fields[i].setLong(str, Long.parseLong( array[i]));
					}
					else if (fieldType.equals("double")){
						fields[i].setDouble(str, Double.parseDouble( array[i]));
					}
					else if (fieldType.equals("float")){
						fields[i].setFloat(str, Float.parseFloat( array[i]));
					}
					else if (fieldType.equals("string")){
						fields[i].set(str, array[i]);
					}
				}		
				
				if( str != null )
				{
					if (sourceObj == null)
						System.out.println("sourceObj == null; " + objectDef.getName());
					else
						System.out.println(objectDef.getName());
					
					scomm.setStructureFields(sourceObj, str, sourceField, strClass);
									
					logFile.println("     " + sourceField + " = " + value);
					logFile.println("     --> setPropertyObjectValue($PARENT," + objectDef.getName() + ",$CHILD," + property.getType() + ")");
				}				
			}
			catch (RuntimeException e) {
				System.err.println( "Importer: cannot set " + property.getName() + 
						" of object " + objectDef.getName() );
			} 
			catch(Exception e){
				e.printStackTrace();
			}	
		}
		else
			logFile.println("!    " + sourceField + " = (undefined)");
			
			
	}
	
	private void setObjectSequence(PropertyDef property, Element xmlObject, Data sourceObj,
			ObjectDef objectDef, SComm scomm){
		
		String sourceField = property.getName();
		String value = getPropValue( xmlObject, property.getName() );
	//	System.out.println("----- " + value + " prop: " + property.getName() );
		if( value != null)
		{
			String[] array = value.split(separate);
			Data[] arrayData = new Data[array.length];
			for (int i = 0; i < array.length; i++){
				value = property.getType() + "." + array[i];
				arrayData[i] = findVarValue(value);
			}
			if (arrayData[0] != null ){
				if (sourceObj == null)
					System.out.println("sourceObj == null; " + objectDef.getName());
				else
					System.out.println(objectDef.getName());
				
				scomm.setIDataArray( sourceObj, arrayData, property );
			}
			else
			{
				logFile.println("!    " + sourceField + " = (cannot find " + value + ")");
			}
		}
		else
			logFile.println("!    " + sourceField + " = (undefined)");		
	}
	
	private void setStructSequence(PropertyDef property, Element xmlObject, Data sourceObj,
			ObjectDef objectDef, SComm scomm){
		
		String sourceField = property.getName();
		String value = getPropValue( xmlObject, property.getName() );
	//	System.out.println("----- " + value + " prop: " + property.getName() );
		if( value != null){
			String[] arrayStructuresValues = value.split(separate);
			ObjectDef strDef = SComm.getObjects().getObjectDef(property.getType());
			String strName = strDef.getClassName();
			
			try{
				Class strClass = Class.forName(strName);
				Object arrayStructuresObjects = Array.newInstance(strClass, arrayStructuresValues.length);
				
				Field[] fields = strClass.getDeclaredFields();
			
				for (int j = 0; j < arrayStructuresValues.length; j++){
					String[] array = arrayStructuresValues[j].split(innerSep);
				
					Object str = strClass.newInstance();
				
					for (int i = 0; i < fields.length; i++){
						String fieldType = fields[i].getType().getSimpleName();
				
						if (ObjectData.isObject(fieldType)){
							Data targetObj =  findVarValue(array[i]);
							if (targetObj != null)
								fields[i].set(str, targetObj);
						}
						else if (fieldType.equals("int")){
							fields[i].setInt(str, Integer.parseInt( array[i]));
						}
						else if (fieldType.equals("long")){
							fields[i].setLong(str, Long.parseLong( array[i]));
						}
						else if (fieldType.equals("double")){
							fields[i].setDouble(str, Double.parseDouble( array[i]));
						}
						else if (fieldType.equals("float")){
							fields[i].setFloat(str, Float.parseFloat( array[i]));
						}
						else if (fieldType.equals("string")){
							fields[i].set(str, array[i]);
						}
					}	
					Array.set(arrayStructuresObjects, j, str);
				}
					
				if( arrayStructuresObjects != null )
				{
					if (sourceObj == null)
						System.out.println("sourceObj == null; " + objectDef.getName());
					else
						System.out.println(objectDef.getName());
										
						
					scomm.setStructureFields(sourceObj, arrayStructuresObjects, sourceField, arrayStructuresObjects.getClass());
										
					logFile.println("     " + sourceField + " = " + value);
					logFile.println("     --> setPropertyObjectValue($PARENT," + objectDef.getName() + ",$CHILD," + property.getType() + ")");
				}				
			}
			catch (RuntimeException e) {
				System.err.println( "Importer: cannot set " + property.getName() + 
						" of object " + objectDef.getName() );
			} 
			catch(Exception e){
				e.printStackTrace();
			}	
			
		}
		else
			logFile.println("!    " + sourceField + " = (undefined)");
	}
	
	private void setReference(PropertyDef property, Element xmlObject, Data sourceObj,
			ObjectDef objectDef, SComm scomm){
				
		String value = getPropValue( xmlObject, property.getName() );
		if (!value.equals("0") || value == null){
			String methodName = "set".concat(property.getName());
			String referenceXmlObjectName = property.getReference() + Exporter.idNumber(Integer.parseInt(value));
			
			try{
				int referenceId = (Integer) table.get(referenceXmlObjectName);
				Class cl = Class.forName(objectDef.getClassName());
				
				for (Method method : cl.getMethods()){
					if (method.getName().equals(methodName)){
						
						method.invoke(sourceObj, referenceId);
						break;
					}
				}				
			}catch (Exception e){
				e.printStackTrace();
			}		
		}
	}
	
	private String getBasicName(Data data, String name){		
		String s = data.getClass().getSuperclass().getSimpleName();
		Class cl = data.getClass();
		Class cl2 = data.getClass().getSuperclass();
		return data.getClass().getSuperclass().getSimpleName() + name.substring(name.length() - 4);
	}

}
